# Authors:
#  Stephen Gallagher <sgallagh@redhat.com>
#
# Copyright (C) 2009  Red Hat
# see file 'COPYING' for use and warranty information
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

"""
Python-level packaging using setuptools.
"""

from setuptools import setup

def sanitize_version(version):
    """
    We need to convert Fedora version guidelines which we follow in version.m4
    to Python guidelines. See:
    * https://docs.fedoraproject.org/en-US/packaging-guidelines/Versioning/
    * https://packaging.python.org/en/latest/discussions/versioning/
    """
    # X.Y.Z-alpha1 -> X.Y.Za1
    # X.Y.Z-beta1 -> X.Y.Zb1
    # X.Y.Z-rc1 -> X.Y.Zrc1
    return version.replace('-', '').replace('alpha', 'a').replace('beta', 'b')

setup(
    name='SSSDConfig',
    version=sanitize_version('2.10.1'),
    license='GPLv3+',
    url='https://github.com/SSSD/sssd/',
    packages=['SSSDConfig'],
)



"""conftest.py for all tests"""

from __future__ import print_function
import os
import time
import posixpath
import ldap
import pytest
import re
import subprocess
import random
from datetime import datetime, timedelta
from constants import ds_instance_name, ds_suffix, krb_realm, ds_rootdn, ds_rootpw
from sssd.testlib.common.libkrb5 import krb5srv
from sssd.testlib.common.paths import SSSD_DEFAULT_CONF, NSSWITCH_DEFAULT_CONF
from sssd.testlib.common.utils import PkiTools, sssdTools, LdapOperations
from sssd.testlib.common.libdirsrv import DirSrvWrap
from sssd.testlib.common.exceptions import PkiLibException, LdapException

pytest_plugins = (
    'sssd.testlib.common.fixtures',
    'pytest_importance',
    'pytest_ticket',
    'sssd.testlib.common.custom_log',
)


def pytest_configure():
    """ Namespace hook to add below dict in the pytest namespace """
    pytest.num_masters = 2
    pytest.num_ad = 0
    pytest.num_atomic = 0
    pytest.num_replicas = 0
    pytest.num_clients = 1
    pytest.num_others = 0

# ==================== Function Scoped Fixtures ==============


@pytest.fixture(scope='function')
def multidomain_sssd(session_multihost, request):
    """ Multidomain sssd configuration """
    session_multihost.client[0].service_sssd('stop')
    tools = sssdTools(session_multihost.client[0])
    tools.backup_sssd_conf()
    tools.remove_sss_cache('/var/lib/sss/db')
    tools.remove_sss_cache('/var/lib/sss/mc')

    def _modifysssd(domains):
        """ Modify sssd.conf """
        if domains == 'proxy_ldap2':
            ds_host = session_multihost.master[1].sys_hostname
            sssd_params = {'domains': 'proxy, ldap2'}
            tools.sssd_conf('sssd', sssd_params)
            ldap_uri = 'ldaps://%s' % ds_host
            suffix = 'dc=example1,dc=test'
            domain_params = {'ldap_search_base': suffix, 'ldap_uri': ldap_uri}
            tools.sssd_conf('domain/ldap2', domain_params)

        if domains == 'local_proxy':
            sssd_params = {'domains': 'proxy, local'}
            tools.sssd_conf('sssd', sssd_params)
            proxy_params = {'min_id': '2000', 'max_id': '2010'}
            local_params = {'min_id': '5000', 'max_id': '5010'}
            tools.sssd_conf('domain/proxy', proxy_params)
            tools.sssd_conf('domain/local', local_params)

        if domains == 'local_ldap':
            ds_host = session_multihost.master[1].sys_hostname
            sssd_params = {'domains': 'local, ldap1'}
            tools.sssd_conf('sssd', sssd_params)
            ldap_uri = 'ldaps://%s' % (ds_host)
            suffix = 'dc=example1,dc=test'
            domain_params = {'ldap_search_base': suffix,
                             'ldap_uri': ldap_uri,
                             'min_id': '3000', 'max_id': '3010'}
            files_params = {'min_id': '2000', 'max_id': '2010'}
            tools.sssd_conf('domain/ldap1', domain_params)
            tools.sssd_conf('domain/local', files_params)

        if domains == 'ldap_ldap':
            sssd_params = {'domains': 'ldap1, ldap2'}
            tools.sssd_conf('sssd', sssd_params)
            id_suffix = ['20', '30']
            for idx in range(2):
                ds_host = session_multihost.master[idx].sys_hostname
                ldap_uri = 'ldaps://%s' % (ds_host)
                suffix = 'dc=example%d,dc=test' % idx
                u_search_base = 'ou=People,%s' % suffix
                g_search_base = 'ou=Groups,%s' % suffix
                ldap_params = {'ldap_search_base': suffix,
                               'ldap_uri': ldap_uri,
                               'min_uid': '%s00' % id_suffix[idx],
                               'max_id': '%s20' % id_suffix[idx],
                               'ldap_group_search_base': g_search_base,
                               'ldap_user_search_base': u_search_base,
                               'cache_credentials': 'False',
                               'use_fully_qualified_names': 'True'}
                domain_section = 'domain/ldap%d' % (idx + 1)
                tools.sssd_conf(domain_section, ldap_params)

    def removesssd():
        """ Remove sssd configuration """
        stop_sssd = 'systemctl stop sssd'
        session_multihost.client[0].run_command(stop_sssd)
        tools.restore_sssd_conf()
    request.addfinalizer(removesssd)
    return _modifysssd


@pytest.fixture(autouse=True)
def capture_sssd_logs(session_multihost, request):
    """This will print sssd logs in case of test failure"""
    yield
    if request.session.testsfailed:
        client = session_multihost.client[0]
        print(f"\n\n===Logs for {request.node.name}===\n\n")
        for data_d in client.run_command("ls /var/log/sssd/").stdout_text.split():
            client.run_command(f'echo "--- {data_d} ---"; '
                               f'cat /var/log/sssd/{data_d}')


@pytest.fixture(scope="function", autouse=True)
def setup_authselect(session_multihost):
    """
    Make sure to use sssd as authselect profile
    """
    # We should not overwrite nsswitch that is changed/pre-configured
    session_multihost.client[0].run_command(
        "test -L /etc/nsswitch.conf && authselect select sssd --force", raiseonerr=False
    )


@pytest.fixture(scope='function')
def ldap_posix_usergroup(session_multihost, request):
    """ Create single ldap posix user group """
    ldap_uri = f'ldap://{session_multihost.master[0].sys_hostname}'
    ldap_inst = LdapOperations(ldap_uri, ds_rootdn, ds_rootpw)
    krb = krb5srv(session_multihost.master[0], 'EXAMPLE.TEST')
    id = random.randint(9, 99)
    user_info = {'cn': f'usr_{id}',
                 'uid': f'usr_{id}',
                 'uidNumber': f'345831{id}',
                 'gidNumber': f'345641{id}'}
    if ldap_inst.posix_user("ou=People", "dc=example,dc=test", user_info):
        krb.add_principal(f'usr_{id}', 'user', 'Secret123')
    else:
        print(f"Unable to add ldap User {user_info}")
        assert False
    memberdn = f'uid=usr_{id},ou=People,dc=example,dc=test'
    group_info = {'cn': f'ldapgrp{id}',
                  'gidNumber': f'345641{id}',
                  'uniqueMember': memberdn}
    try:
        ldap_inst.posix_group("ou=Groups", "dc=example,dc=test", group_info)
    except LdapException:
        assert False

    def delposixobject():
        """ Delete ldap posix user and group """
        ldap_inst.del_dn(f'uid=usr_{id},ou=People,dc=example,dc=test')
        ldap_inst.del_dn(f'cn=ldapgrp{id},ou=Groups,dc=example,dc=test')
        krb.delete_principal(f'usr_{id}')
    request.addfinalizer(delposixobject)
    return f'usr_{id}'


@pytest.fixture(scope='function')
def localusers(session_multihost, request):
    """ Create local users """
    users = {'user5000': '5000', 'user5001': '5001'}
    for key, value in users.items():
        useradd = 'useradd -u %s %s' % (value, key)
        session_multihost.client[0].run_command(useradd, raiseonerr=False)
        passwd = 'passwd --stdin %s' % (key)
        session_multihost.client[0].run_command(passwd,
                                                stdin_text='Secret123',
                                                raiseonerr=False)

    def delusers():
        """ Delete local users """
        for key, _ in users.items():
            userdel = 'userdel -f -r %s' % key
            session_multihost.client[0].run_command(userdel)
    request.addfinalizer(delusers)
    return users


@pytest.fixture(scope='function')
def create_350_posix_users(session_multihost, request):
    """ Create posix user and groups """
    ldap_uri = 'ldap://%s' % (session_multihost.master[0].sys_hostname)
    ds_rootdn = 'cn=Directory Manager'
    ds_rootpw = 'Secret123'
    ldap_inst = LdapOperations(ldap_uri, ds_rootdn, ds_rootpw)
    krb = krb5srv(session_multihost.master[0], 'EXAMPLE.TEST')
    for i in range(1, 351):
        user_info = {'cn': 'doo%d' % i,
                     'uid': 'doo%d' % i,
                     'uidNumber': '145831%d' % i,
                     'gidNumber': f'145641{i}'}
        if ldap_inst.posix_user("ou=People",
                                "dc=example,dc=test",
                                user_info):
            krb.add_principal('doo%d' % i, 'user', 'Secret123')
        else:
            print("Unable to add ldap User %s" % (user_info))
            assert False

    def remove_users():
        """ Remove default sssd.conf """
        for i in range(1, 351):
            ldap_inst.del_dn(f'uid=doo{i},ou=People,dc=example,dc=test')
            krb.delete_principal(f'doo{i}')

    request.addfinalizer(remove_users)


@pytest.fixture(scope='function')
def enable_sss_sudo_nsswitch(session_multihost, request):
    """Enable sss backend for sudoers in nsswitch.conf """
    distro = session_multihost.client[0].distro
    if 'Fedora' or '8.' in distro:
        cmd = 'authselect select sssd with-sudo'
        session_multihost.client[0].run_command(cmd)
    else:
        backup = 'cp -af /etc/nsswitch.conf /etc/nsswitch.conf.backup'
        session_multihost.client[0].run_command(backup)
        content = '\nsudoers: files sss\n'
        session_multihost.client[0].put_file_contents(content,
                                                      NSSWITCH_DEFAULT_CONF)

    def restore_nsswitch():
        """ Restore nsswitch.conf """
        if 'Fedora' or '8.' in distro:
            cmd = 'authselect select sssd'
            session_multihost.client[0].run_command(cmd)
        else:
            restore = 'cp -f /etc/nsswitch.conf.backup /etc/nsswitch.conf'
            session_multihost.client[0].run_command(restore)
    request.addfinalizer(restore_nsswitch)


@pytest.fixture(scope='function')
def backupsssdconf(session_multihost, request):
    """ Backup and restore sssd.conf """
    tools = sssdTools(session_multihost.client[0])
    tools.backup_sssd_conf()
    session_multihost.client[0].service_sssd('stop')

    def restoresssdconf():
        """ Restore sssd.conf """
        tools.restore_sssd_conf()
    request.addfinalizer(restoresssdconf)


@pytest.fixture(scope='function')
def delete_groups_users(session_multihost, request):
    """Fixture for bz1817122"""
    ldap_uri = 'ldap://%s' % (session_multihost.master[0].sys_hostname)
    ds_rootdn = 'cn=Directory Manager'
    ds_rootpw = 'Secret123'
    ldap_inst = LdapOperations(ldap_uri, ds_rootdn, ds_rootpw)

    def restoresssdconf():
        """Delete users, ous and groups"""
        for i in range(1, 9):
            ldap_inst.del_dn(f'cn=user-{i},ou=posix_groups,ou=Unit2,'
                             f'ou=Unit1,dc=example,dc=test')
        for i in range(1, 3):
            ldap_inst.del_dn(f'cn=group-{i},ou=posix_groups,ou=Unit2,'
                             f'ou=Unit1,dc=example,dc=test')
        for i in range(1, 9):
            ldap_inst.del_dn(f'cn=user-{i},ou=users,ou=Unit2,'
                             f'ou=Unit1,dc=example,dc=test')
        for dn_dn in ['netgroups', 'services', 'sudoers']:
            ldap_inst.del_dn(f'ou={dn_dn},dc=example,dc=test')
        for dn_dn in ['ou=posix_groups,ou=Unit2,ou=Unit1,dc=example,dc=test',
                      'ou=users,ou=Unit2,ou=Unit1,dc=example,dc=test',
                      'ou=Unit2,ou=Unit1,dc=example,dc=test',
                      'ou=Unit1,dc=example,dc=test']:
            ldap_inst.del_dn(dn_dn)
    request.addfinalizer(restoresssdconf)


@pytest.fixture(scope="function")
def set_dslimits(session_multihost, request):
    """ Modify nsslapd-sizelimit """
    ldap_uri = 'ldap://%s' % (session_multihost.master[0].sys_hostname)
    ds_rootdn = 'cn=Directory Manager'
    ds_rootpw = 'Secret123'
    ldap_inst = LdapOperations(ldap_uri, ds_rootdn, ds_rootpw)
    configdn = 'cn=config,cn=ldbm database,cn=plugins,cn=config'
    mod_limit = [(ldap.MOD_REPLACE, 'nsslapd-lookthroughlimit', [b'10'])]
    (ret, _) = ldap_inst.modify_ldap(configdn, mod_limit)
    mod_limit = [(ldap.MOD_REPLACE, 'nsslapd-pagedlookthroughlimit', [b'10'])]
    (ret, _) = ldap_inst.modify_ldap(configdn, mod_limit)

    assert ret == 'Success'

    def restore_dslimits():
        """ Restore the default sizelimit """
        restore_lookthrough = [(ldap.MOD_REPLACE, 'nsslapd-lookthroughlimit',
                               [b'2000'])]
        (ret, _) = ldap_inst.modify_ldap(configdn, restore_lookthrough)
        restore_page = [(ldap.MOD_REPLACE,
                         'nsslapd-pagedlookthroughlimit', [b'2000'])]
        (ret, _) = ldap_inst.modify_ldap(configdn, restore_page)
        assert ret == 'Success'
    request.addfinalizer(restore_dslimits)


@pytest.fixture(scope="function")
def add_nisobject(session_multihost, request):
    """ Add nisobject to auto.direct map """
    share_list = [request.param]
    nfs_server = session_multihost.master[0].external_hostname
    client_ip = session_multihost.client[0].ip
    server = sssdTools(session_multihost.master[0])
    bkup = 'cp -af /etc/exports /etc/exports.backup'
    session_multihost.master[0].run_command(bkup)
    server.export_nfs_fs(share_list, client_ip)
    start_nfs = 'systemctl start nfs-server'
    try:
        session_multihost.master[0].run_command(start_nfs)
    except subprocess.CalledProcessError:
        pytest.fail("Unable to start nfs server")
    ldap_uri = 'ldap://%s' % (session_multihost.master[0].sys_hostname)
    ds_rootdn = 'cn=Directory Manager'
    ds_rootpw = 'Secret123'
    ds_suffix = 'dc=example,dc=test'
    ldap_inst = LdapOperations(ldap_uri, ds_rootdn, ds_rootpw)
    ret = ldap_inst.add_map(request.param, 'auto.direct',
                            nfs_server, request.param, ds_suffix)
    assert ret == 'Success'

    def remove_project():
        """ Remove the nisproject from map """
        stop_autofs = 'systemctl stop autofs'
        session_multihost.client[0].run_command(stop_autofs)
        ret = ldap_inst.delete_map(request.param, 'auto.direct', ds_suffix)
        assert ret == 'Success'
        remove_share = 'rm -rf %s' % request.param
        session_multihost.master[0].run_command(remove_share)
        restore = 'cp -f /etc/exports.backup /etc/exports'
        session_multihost.master[0].run_command(restore)
        stop_nfs = "systemctl stop nfs-server"
        try:
            session_multihost.master[0].run_command(stop_nfs)
        except subprocess.CalledProcessError:
            pytest.fail("failed to stop nfs-server")
    request.addfinalizer(remove_project)


@pytest.fixture(scope='function')
def create_etc_exports(session_multihost, request):
    """ Remove and recreate /etc/exports """
    remove_exports = 'rm -f /etc/exports'
    session_multihost.master[0].run_command(remove_exports)
    # create an empty /etc/exports
    create_exports_file = 'touch /etc/exports'
    session_multihost.master[0].run_command(create_exports_file)


@pytest.fixture(scope='function')
def indirect_nismaps(session_multihost, request, create_etc_exports):
    """ Create indirect map and add 20 map keys.
        Also create /projects/foo1 to /projects/foo20
        Restart nfs server
    """
    nfs_server = session_multihost.master[0].external_hostname
    client_ip = session_multihost.client[0].ip
    server = sssdTools(session_multihost.master[0])
    ldap_uri = 'ldap://%s' % (session_multihost.master[0].sys_hostname)
    ds_rootdn = 'cn=Directory Manager'
    ds_rootpw = 'Secret123'
    ldap_inst = LdapOperations(ldap_uri, ds_rootdn, ds_rootpw)
    server.export_nfs_fs(['/projects'], client_ip)
    for i in range(1, 20):
        map_keys = ['foo%d' % (i)]
        server_path = '/projects/foo%d' % (i)
        ret = ldap_inst.add_map(map_keys[0], 'auto.idmtest',
                                nfs_server, server_path, ds_suffix)
        permissions = '(rw,sync)'
        path_list = ['/projects/%s' % map_keys[0]]
        server.export_nfs_fs(path_list, client_ip, permissions)
        assert ret == 'Success'
    start_nfs = 'systemctl start nfs-server'
    export_fs = 'exportfs -a'
    try:
        session_multihost.master[0].run_command(export_fs)
    except subprocess.CalledProcessError:
        pytest.fail("Unable to start nfs server")
    try:
        session_multihost.master[0].run_command(start_nfs)
    except subprocess.CalledProcessError:
        pytest.fail("Unable to start nfs server")

    def remove_bulk_maps():
        """ Remove the nisproject from map """
        stop_autofs = 'systemctl stop autofs'
        session_multihost.client[0].run_command(stop_autofs)
        for i in range(1, 20):
            map_keys = ['foo%d' % (i)]
            ret = ldap_inst.delete_map(map_keys[0], 'auto.idmtest', ds_suffix)
            assert ret == 'Success'
        # remove /projects
        remove_projects = 'rm -rf /projects'
        session_multihost.master[0].run_command(remove_projects)
        restore = 'cp -f /etc/exports.backup /etc/exports'
        session_multihost.master[0].run_command(restore)
        stop_nfs = "systemctl stop nfs-server"
        try:
            session_multihost.master[0].run_command(stop_nfs)
        except subprocess.CalledProcessError:
            pytest.fail("failed to stop nfs-server")
    request.addfinalizer(remove_bulk_maps)


@pytest.fixture(scope="function")
def set_autofs_search_base(session_multihost, request):
    """ Enable autofs service """
    client = sssdTools(session_multihost.client[0])
    client.backup_sssd_conf()
    domain_name = client.get_domain_section_name()
    domain_section = 'domain/{}'.format(domain_name)
    autofs_dn = "ou=automount,%s" % (ds_suffix)
    sssd_params = {'ldap_autofs_search_base': autofs_dn}
    client.sssd_conf(domain_section, sssd_params)

    def remove_autofs_search_base():
        """ Remove autofs search base """
        client.restore_sssd_conf()
        client.clear_sssd_cache()
    request.addfinalizer(remove_autofs_search_base)


@pytest.fixture(scope="function")
def set_ldap_uri(session_multihost, request):
    """ Replace ldaps uri with ldap uri """
    tools = sssdTools(session_multihost.client[0])
    ldap_uri = 'ldap://%s' % session_multihost.master[0].sys_hostname
    ldap_params = {'ldap_uri': ldap_uri}
    tools.sssd_conf('domain/%s' % (ds_instance_name), ldap_params)
    session_multihost.client[0].service_sssd('restart')

    def restore_ldaps_uri():
        """ Restore ldaps uri """
        ldap_uri = 'ldaps://%s' % session_multihost.master[0].sys_hostname
        ldap_params = {'ldap_uri': ldap_uri}
        tools.sssd_conf('domain/%s' % (ds_instance_name), ldap_params)
        session_multihost.client[0].service_sssd('restart')
    request.addfinalizer(restore_ldaps_uri)


@pytest.fixture(scope="function")
def create_ssh_keys(session_multihost, request):
    """ Create ssh keys """
    ldap_uri = 'ldap://%s' % (session_multihost.master[0].sys_hostname)
    ds_rootdn = 'cn=Directory Manager'
    ds_rootpw = 'Secret123'
    ldap_inst = LdapOperations(ldap_uri, ds_rootdn, ds_rootpw)
    keygen_cmd = "ssh-keygen -q -t rsa -b 2048 -N '' -f /tmp/id_rsa -C ''"
    session_multihost.client[0].run_command(keygen_cmd)
    get_pub_key = 'cat /tmp/id_rsa.pub'
    cmd = session_multihost.client[0].run_command(get_pub_key)
    ssh_key = cmd.stdout_text.strip().split('\n')[0]
    user_dn = "uid=foo1,ou=People,%s" % ds_suffix
    add_objectclass = [(ldap.MOD_ADD, 'objectClass', [b'ldapPublicKey'])]
    (ret, _) = ldap_inst.modify_ldap(user_dn, add_objectclass)
    assert ret == 'Success'
    add_pubkey = [(ldap.MOD_ADD, 'sshPublicKey', [ssh_key.encode('utf-8')])]
    (ret, _) = ldap_inst.modify_ldap(user_dn, add_pubkey)
    assert ret == 'Success'

    def remove_keys():
        """ Remove ssh key file """
        remove_ssh = 'rm -f /tmp/id_rsa*'
        session_multihost.client[0].run_command(remove_ssh)
    request.addfinalizer(remove_keys)


@pytest.fixture(scope='function')
def enable_multiple_responders(session_multihost, request):
    """ Enable multiple responders to sssd services """
    session_multihost.client[0].service_sssd('stop')
    tools = sssdTools(session_multihost.client[0])
    services = 'nss, pam, sudo, autofs, ssh, pac, ifp'
    srv_list = [x.strip() for x in services.split(',')]
    sssd_params = {'services': services}
    tools.sssd_conf('sssd', sssd_params, action='update')
    param = {'debug_level': '9'}
    for new_sec in srv_list:
        tools.sssd_conf(new_sec, param)
    ret = session_multihost.client[0].service_sssd('start')
    time.sleep(10)
    assert ret == 0

    def restore_sssd_conf():
        """ Restore sssd.conf """
        services = 'nss, pam'
        sssd_params = {'services': services}
        tools.sssd_conf('sssd', sssd_params, action='update')
    request.addfinalizer(restore_sssd_conf)


@pytest.fixture(scope='function')
def sssd_sudo_conf(session_multihost, request):
    """ Configure basic sudo parameters in sssd.conf """
    tools = sssdTools(session_multihost.client[0])
    session_multihost.client[0].service_sssd('stop')
    tools.remove_sss_cache('/var/lib/sss/db/')
    tools.remove_sss_cache('/var/log/sssd')
    section = "sssd"
    sssd_params = {'services': 'nss, pam, sudo'}
    tools.sssd_conf(section, sssd_params)
    sudo_base = f'ou=sudoers,{ds_suffix}'
    params = {'ldap_sudo_search_base': sudo_base,
              'sudo_provider': 'ldap'}
    domain_section = f'domain/{ds_instance_name}'
    tools.sssd_conf(domain_section, params, action='update')
    session_multihost.client[0].service_sssd('start')

    def restore_sssd_conf():
        """ Restore sssd.conf """
        services = 'nss, pam'
        sssd_params = {'services': services}
        tools.sssd_conf('sssd', sssd_params)
        tools.sssd_conf(domain_section, params, action='delete')
    request.addfinalizer(restore_sssd_conf)


@pytest.fixture(scope='function')
def sudo_rule(session_multihost, request):
    """ Create sudoers ldap entries """
    ldap_uri = f'ldap://{session_multihost.master[0].sys_hostname}'
    sudo_ou = f'ou=sudoers,{ds_suffix}'
    ds_rootdn = 'cn=Directory Manager'
    ds_rootpw = 'Secret123'
    ldap_inst = LdapOperations(ldap_uri, ds_rootdn, ds_rootpw)
    try:
        ldap_inst.org_unit('sudoers', ds_suffix)
    except LdapException:
        pytest.fail("already exist or failed to add sudo ou ")
    sudo_options = ["!requiretty", "!authenticate"]
    sudo_cmd = '/usr/bin/head'
    sudo_user = 'foo1'
    rule_dn = f'cn={sudo_cmd},{sudo_ou}'
    try:
        ldap_inst.add_sudo_rule(rule_dn, 'ALL', '/usr/bin/head',
                                sudo_user, sudo_options)
    except LdapException:
        pytest.fail(f"Failed to add sudo rule {rule_dn}")
    else:
        extra_user = 'foo2'
        add_extra = [(ldap.MOD_ADD, 'sudoUser',
                     extra_user.encode('utf-8'))]
        (ret, _) = ldap_inst.modify_ldap(rule_dn, add_extra)
        assert ret == 'Success'

    def del_sudo_rule():
        """ Delete sudo rule  """
        rule_dn = f'cn={sudo_cmd},{sudo_ou}'
        (_, _) = ldap_inst.del_dn(rule_dn)
        (ret, _) = ldap_inst.del_dn(sudo_ou)
        assert ret == 'Success'
    request.addfinalizer(del_sudo_rule)


testdata = [
    [(datetime.today() - timedelta(days=1)).strftime('%Y%m%d%H') + 'Z',
     'sudoNotBefore'],
    [(datetime.today() + timedelta(days=1)).strftime('%Y%m%d%H') + 'Z',
     'sudoNotAfter']]


@pytest.fixture(ids=["sudoNotBefore", "sudoNotAfter"], params=testdata)
def timed_sudoers(session_multihost, request):
    """ Creates a time sudoers ldap entries """
    ldap_uri = 'ldap://%s' % (session_multihost.master[0].sys_hostname)
    sudo_ou = 'ou=sudoers, %s' % ds_suffix
    ds_rootdn = 'cn=Directory Manager'
    ds_rootpw = 'Secret123'
    ldap_inst = LdapOperations(ldap_uri, ds_rootdn, ds_rootpw)
    try:
        ldap_inst.org_unit('sudoers', ds_suffix)
    except LdapException:
        pytest.fail("already exist or failed to add sudo ou ")
    sudo_options = ["!requiretty", "!authenticate"]
    rule_dn = "cn=%s, %s" % (request.param[1], sudo_ou)
    try:
        ldap_inst.add_sudo_rule(rule_dn, 'ALL', '/usr/bin/head',
                                'foo1', sudo_options)
    except LdapException:
        pytest.fail("Failed to add sudo rule %s" % rule_dn)
    else:
        sudotime = request.param[0]
        add_attr = [(ldap.MOD_ADD, request.param[1],
                     sudotime.encode('utf-8'))]
        (ret, _) = ldap_inst.modify_ldap(rule_dn, add_attr)
        assert ret == 'Success'

    def del_sudo_rule():
        """ Delete sudo rule  """
        rule_dn = 'cn=%s,%s' % (request.param[1], sudo_ou)
        (_, _) = ldap_inst.del_dn(rule_dn)
        (ret, _) = ldap_inst.del_dn(sudo_ou)
        assert ret == 'Success'
    request.addfinalizer(del_sudo_rule)

# ====================  Class Scoped Fixtures ================


@pytest.fixture(scope='class')
def setupds(session_multihost, request):
    """ Setup Directory Server """
    server_list = [session_multihost.master[0].sys_hostname]
    pki_inst = PkiTools()
    try:
        certdb = pki_inst.createselfsignedcerts(server_list)
    except PkiLibException:
        pytest.fail("Failed to create CA")
    master = DirSrvWrap(session_multihost.master[0],
                        client_obj=session_multihost.client[0],
                        ssl=True,
                        ssldb=certdb)
    master.create_ds_instance(ds_instance_name, ds_suffix)

    def removeds():
        """ Remove Directory server instance """
        master.remove_ds_instance(ds_instance_name)
    request.addfinalizer(removeds)
    return master


@pytest.fixture(scope='class')
def multipleds(session_multihost, request):
    """ Setup Multiple Directory Servers """
    server_list = [session_multihost.master[0].sys_hostname,
                   session_multihost.master[1].sys_hostname]
    pki_inst = PkiTools()
    try:
        certdb = pki_inst.createselfsignedcerts(server_list)
    except PkiLibException:
        pytest.fail("Failed to create CA")
    print(certdb)
    dsobjlist = []
    for idx in range(2):
        host = session_multihost.master[idx]
        dsobj = DirSrvWrap(host,
                           client_obj=session_multihost.client[0],
                           ssl=True,
                           ssldb=certdb)
        dsobjlist.append(dsobj)
        inst_name = 'example%d' % idx
        suffix = 'dc=example%d,dc=test' % idx
        dsobj.create_ds_instance(inst_name, suffix)

    def removeds():
        """ Remove DS Instances """
        idx = 0
        for dsinst in dsobjlist:
            instname = 'example%d' % idx
            dsinst.remove_ds_instance(instname)
            idx += 1

    request.addfinalizer(removeds)


@pytest.fixture(scope='class')
def multipleds_failover(session_multihost, request):
    """ Setup Multiple Directory Servers for failover"""
    server_list = [session_multihost.master[0].sys_hostname,
                   session_multihost.master[1].sys_hostname]
    pki_inst = PkiTools()
    try:
        certdb = pki_inst.createselfsignedcerts(server_list)
    except PkiLibException:
        pytest.fail("Failed to create CA")
    print(certdb)
    dsobjlist = []
    for idx in range(2):
        host = session_multihost.master[idx]
        dsobj = DirSrvWrap(host,
                           client_obj=session_multihost.client[0],
                           ssl=True,
                           ssldb=certdb)
        dsobjlist.append(dsobj)
        inst_name = 'example'
        suffix = 'dc=example,dc=test'
        dsobj.create_ds_instance(inst_name, suffix)

    def removeds():
        """ Remove DS Instances """
        for dsinst in dsobjlist:
            instname = 'example'
            dsinst.remove_ds_instance(instname)
    request.addfinalizer(removeds)


@pytest.fixture(scope='class')
# pylint: disable=unused-argument
def posix_users_multidomain(session_multihost, multipleds):
    """ Create posix users groups for multidomain """
    suffix = ['p', 'q']
    id_suffix = ['20', '30']
    for idx in range(2):
        host = session_multihost.master[idx]
        ldap_uri = 'ldap://%s' % (host.sys_hostname)
        ds_rootdn = 'cn=Directory Manager'
        ds_rootpw = 'Secret123'
        ds_suffix = 'dc=example%d,dc=test' % idx
        ldap_inst = LdapOperations(ldap_uri, ds_rootdn, ds_rootpw)
        for i in range(20):
            uid = "{:02d}".format(i)
            user_info = {'cn': '%suser%d' % (suffix[idx], i),
                         'uid': '%suser%d' % (suffix[idx], i),
                         'uidNumber': '%s%s' % (id_suffix[idx], uid),
                         'gidNumber': '%s%s' % (id_suffix[idx], uid)}
            ldap_inst.posix_user("ou=People", ds_suffix, user_info)
            user = '%suser%d' % (suffix[idx], i)
            memberdn = 'uid=%s,ou=People,%s' % (user, ds_suffix)
            group_info = {'cn': '%sgroup%d' % (suffix[idx], i),
                          'gidNumber': '%s%s' % (id_suffix[idx], uid),
                          'uniqueMember': memberdn}
            try:
                ldap_inst.posix_group("ou=Groups", ds_suffix, group_info)
            except LdapException:
                raise
        duplicate_group = {'cn': 'duplicate',
                           'gidNumber': '%s%s' % (id_suffix[idx], uid),
                           'uniqueMember': memberdn}
        try:
            ldap_inst.posix_group("ou=Groups", ds_suffix, duplicate_group)
        except LdapException:
            raise


@pytest.fixture(scope='class')
def sssdproxyldap(session_multihost, request):
    """ Copy sssd proxy pam ldap config file """
    cwd = os.path.dirname(os.path.abspath(__file__))
    remote = '/etc/pam.d/sssdproxyldap'
    source = posixpath.join(cwd, 'sssdproxyldap')
    session_multihost.client[0].transport.put_file(source, remote)

    def removeproxyldap():
        """ Remove sssd proxy pam ldap config file """
        cmd = 'rm -f %s' % remote
        session_multihost.client[0].run_command(cmd)
    request.addfinalizer(removeproxyldap)


@pytest.fixture(scope='class')
def install_nslcd(session_multihost, request):
    """ Install nss-pam-ldapd Configure nslcd.conf """
    client = session_multihost.client[0]
    client.run_command("yum install -y nss-pam-ldapd")
    execute_cmd(session_multihost, "echo 'uid nslcd' > /etc/nslcd.conf")
    execute_cmd(session_multihost, "echo 'gid ldap' >> /etc/nslcd.conf")
    execute_cmd(session_multihost, f"echo 'uri ldap://"
                f"{session_multihost.master[0].ip}'"
                f" >> /etc/nslcd.conf")
    execute_cmd(session_multihost, f"echo 'base {ds_suffix}' >> "
                f"/etc/nslcd.conf")
    execute_cmd(session_multihost, "systemctl restart nslcd")

    def restore_install_nslcd():
        """ Restore"""
        client.run_command("rm -vf /etc/nslcd.conf")
        execute_cmd(session_multihost, "systemctl stop nslcd")

    request.addfinalizer(restore_install_nslcd)


@pytest.fixture(scope='class')
def sssdproxyldap_test(session_multihost, request):
    """ Configure  sssdproxyldap
        Configure  sssd.conf
        Transport sssdproxyldap.sh to client machine
        configure password for ldap user
    """
    master = session_multihost.master[0]
    client = session_multihost.client[0]
    tools = sssdTools(session_multihost.client[0])
    domain_name = tools.get_domain_section_name()
    domain_params = {'proxy_pam_target': 'sssdproxyldap',
                     'id_provider': 'proxy',
                     'proxy_lib_name': 'ldap'}
    tools.sssd_conf('domain/' + domain_name, domain_params)
    file_location = '/script/sssdproxyldap.sh'
    client.transport.put_file(os.path.dirname(os.path.abspath(__file__))
                              + file_location,
                              '/tmp/sssdproxyldap.sh')
    execute_cmd(session_multihost, "chmod 755 /tmp/sssdproxyldap.sh")
    master.run_command("kadmin.local -q "
                       "'addprinc -pw Secret123 "
                       "foo2@example1'")
    tools.clear_sssd_cache()

    def restore_sssdproxyldap_test():
        """ Restore"""
        client.run_command("rm -vf /tmp/sssdproxyldap.sh")
    request.addfinalizer(restore_sssdproxyldap_test)


@pytest.fixture(scope='class')
def nslcd(session_multihost, request):
    """ Setup nslcd.conf """
    ldap_uri = session_multihost.master[0].sys_hostname
    session_multihost.client[0].run_command("yum install "
                                            "-y nss-pam-ldapd",
                                            raiseonerr=False)
    basedn = 'dc=example0,dc=test'
    contents = '''uid nslcd
gid ldap
uri ldap://%s
base %s
''' % (ldap_uri, basedn)
    session_multihost.client[0].put_file_contents('/etc/nslcd.conf', contents)
    start_nslcd = 'systemctl start nslcd'
    session_multihost.client[0].run_command(start_nslcd)

    def remove_nslcd():
        """ Remove nslcd """
        cmd = 'rm -f /etc/nslcd.conf'
        stop_nslcd = 'systemctl stop nslcd'
        session_multihost.client[0].run_command(stop_nslcd)
        session_multihost.client[0].run_command(cmd)
    request.addfinalizer(remove_nslcd)


@pytest.fixture(scope='class')
def template_sssdconf(session_multihost, request):
    """ Copy template sssd conf for multidomain tests """
    cwd = os.path.dirname(os.path.abspath(__file__))
    remote = SSSD_DEFAULT_CONF
    source = posixpath.join(cwd, 'sssd_multidomain.conf')
    session_multihost.client[0].transport.put_file(source, remote)
    tools = sssdTools(session_multihost.client[0])
    tools.fix_sssd_conf_perms()


    def remove_template():
        """ Remove template sssd.conf """
        cmd = f'rm -f {SSSD_DEFAULT_CONF}'
        session_multihost.client[0].run_command(cmd)
    request.addfinalizer(remove_template)


@pytest.fixture(scope="class")
def setup_kerberos(session_multihost, request):
    """ Setup kerberos """
    tools = sssdTools(session_multihost.master[0])
    backup_krb5_conf = 'cp -f /etc/krb5.conf /etc/krb5.conf.orig'
    session_multihost.master[0].run_command(backup_krb5_conf)
    tools.config_etckrb5('EXAMPLE.TEST')
    tools.enable_kcm()
    krb = krb5srv(session_multihost.master[0], 'EXAMPLE.TEST')
    krb.krb_setup_new()

    def remove_kerberos():
        """ Remove kerberos instance """
        krb.destroy_krb5server()
        remove_keytab = 'rm -f /etc/krb5.keytab'
        session_multihost.master[0].run_command(remove_keytab)
        restore_krb5 = 'cp -f /etc/krb5.conf.orig /etc/krb5.conf'
        session_multihost.master[0].run_command(restore_krb5)
    request.addfinalizer(remove_kerberos)


@pytest.fixture(scope="class")
def setup_ds_sasl(session_multihost, request):
    """  Enable sasl on Directory Server  """
    ds_hostname = session_multihost.master[0].sys_hostname
    krb = krb5srv(session_multihost.master[0], 'EXAMPLE.TEST')
    krb.add_principal(p_type=None, service='ldap', service_name=ds_hostname)
    ldap_princ = "ldap/%s" % ds_hostname
    krb.add_remove_keytab(ldap_princ, "/etc/dirsrv/krb5.keytab", 'add')
    keytab_perms = "chown dirsrv.dirsrv /etc/dirsrv/krb5.keytab"
    session_multihost.master[0].run_command(keytab_perms)
    cmd = "echo -e KRB5_KTNAME=/etc/dirsrv/krb5.keytab"\
          " >> /etc/sysconfig/dirsrv-%s" % ds_instance_name
    session_multihost.master[0].run_command(cmd)
    # restart dirsrv
    restart_ds = 'systemctl restart dirsrv@%s' % ds_instance_name
    session_multihost.master[0].run_command(restart_ds)

    def remove_ds_sasl():
        """ Remove keytab file """
        remove_ds_keytab = 'rm -f /etc/dirsrv/krb5.keytab'
        session_multihost.master[0].run_command(remove_ds_keytab)
    request.addfinalizer(remove_ds_sasl)


@pytest.fixture(scope='class')
def setup_sssd(session_multihost, setupds,  # pylint: disable=unused-argument
               setup_kerberos, request):  # pylint: disable=unused-argument
    """ Configure sssd.conf """
    backup_krb5 = "cp -f /etc/krb5.conf /etc/krb5.conf.orig"
    session_multihost.client[0].run_command(backup_krb5)
    tools = sssdTools(session_multihost.client[0])
    krb5_server = session_multihost.master[0].sys_hostname
    tools.config_etckrb5('EXAMPLE.TEST', krb5_server)
    tools.enable_kcm()
    ds_host = session_multihost.master[0].sys_hostname
    sssd_params = {'domains': ds_instance_name}
    tools.sssd_conf('sssd', sssd_params)
    domain_section = 'domain/%s' % ds_instance_name
    ldap_uri = 'ldaps://%s' % (ds_host)
    domain_params = {'ldap_search_base': ds_suffix,
                     'id_provider': 'ldap',
                     'auth_provider': 'ldap',
                     'ldap_user_home_directory': "/home/%u",
                     'ldap_uri': ldap_uri,
                     'ldap_tls_cacert': '/etc/openldap/cacerts/cacert.pem',
                     'use_fully_qualified_names': 'True',
                     'debug_level': '9'}
    tools.sssd_conf(domain_section, domain_params)
    tools.clear_sssd_cache()
    start_sssd = 'systemctl start sssd'
    session_multihost.client[0].run_command(start_sssd)

    def removesssd():
        """ Remove sssd configuration """
        stop_sssd = 'systemctl stop sssd'
        session_multihost.client[0].run_command(stop_sssd)
        removeconf = 'rm -f %s' % (SSSD_DEFAULT_CONF)
        session_multihost.client[0].run_command(removeconf)
        # revert the krb5.conf
        restore_krb5 = 'cp -f /etc/krb5.conf.orig /etc/krb5.conf'
        session_multihost.client[0].run_command(restore_krb5)
    request.addfinalizer(removesssd)


@pytest.fixture(scope='class')
def setup_sssd_krb(session_multihost, setup_sssd):
    """ Configure sssd.conf with auth_provider = krb5 """
    tools = sssdTools(session_multihost.client[0])
    domain_section = 'domain/%s' % ds_instance_name
    krb5_server = session_multihost.master[0].sys_hostname
    domain_params = {'auth_provider': 'krb5',
                     'krb5_realm': 'EXAMPLE.TEST',
                     'krb5_server': krb5_server}
    tools.sssd_conf(domain_section, domain_params)
    restart_sssd = 'systemctl restart sssd'
    session_multihost.client[0].run_command(restart_sssd,
                                            raiseonerr=False)


@pytest.fixture(scope='class')
def create_host_keytab(session_multihost, request):
    """ Configure host keytab on client """
    # delete keytab file if it exists
    delete_keytab = "rm -f /etc/krb5.keytab"
    session_multihost.client[0].run_command(delete_keytab, raiseonerr=False)
    client_host = session_multihost.client[0].sys_hostname
    add_principal = "add_principal -clearpolicy"\
                    " -randkey host/%s" % client_host
    cmd1 = "kadmin -p root/admin -w Secret123 -q '%s'" % add_principal
    session_multihost.client[0].run_command(cmd1, raiseonerr=False)
    ktadd = 'ktadd -k /etc/krb5.keytab host/%s' % client_host
    cmd2 = "kadmin -p root/admin -w Secret123 -q '%s'" % ktadd
    session_multihost.client[0].run_command(cmd2)

    def remove_keytab():
        """ Remove keytab """
        session_multihost.client[0].run_command(delete_keytab)
    request.addfinalizer(remove_keytab)


@pytest.fixture(scope='class')
def setup_sssd_gssapi(session_multihost, setup_sssd,
                      setup_ds_sasl, create_host_keytab):
    """ Configure sssd.conf  with sasl_mech gssapi """
    tools = sssdTools(session_multihost.client[0])
    domain_section = 'domain/%s' % ds_instance_name
    krb5_server = session_multihost.master[0].sys_hostname
    ldap_uri = 'ldap://%s' % (krb5_server)
    ds_rootdn = 'cn=Directory Manager'
    ds_rootpw = 'Secret123'
    ldap_inst = LdapOperations(ldap_uri, ds_rootdn, ds_rootpw)
    domain_params = {'auth_provider': 'krb5',
                     'ldap_sasl_mech': 'GSSAPI',
                     'krb5_realm': 'EXAMPLE.TEST',
                     'use_fully_qualified_names': 'False',
                     'krb5_server': krb5_server}
    host_info = {'cn': 'host/%s' % session_multihost.client[0].sys_hostname,
                 'uid': 'host/%s' % session_multihost.client[0].sys_hostname,
                 'uidNumber': '19999999',
                 'gidNumber': '14564100'}
    ldap_inst.posix_user("ou=People", "%s" % ds_suffix, host_info)
    tools.sssd_conf(domain_section, domain_params)
    session_multihost.client[0].service_sssd('restart')


@pytest.fixture(scope='class')
def setup_sssd_failover(session_multihost, request):
    """ Configure sssd.conf """
    tools = sssdTools(session_multihost.client[0])
    stop_sssd = 'systemctl stop sssd'
    session_multihost.client[0].run_command(stop_sssd)
    ds_host1 = session_multihost.master[0].sys_hostname
    ds_host2 = session_multihost.master[1].sys_hostname
    sssd_params = {'domains': ds_instance_name}
    tools.sssd_conf('sssd', sssd_params)
    domain_section = 'domain/%s' % ds_instance_name
    ldap_uri = 'ldaps://%s, ldaps://%s' % (ds_host1, ds_host2)
    domain_params = {'ldap_search_base': ds_suffix,
                     'id_provider': 'ldap',
                     'auth_provider': 'ldap',
                     'ldap_user_home_directory': "/home/%u",
                     'ldap_uri': ldap_uri,
                     'ldap_tls_cacert': '/etc/openldap/cacerts/cacert.pem',
                     'use_fully_qualified_names': 'True',
                     'debug_level': '9'}
    tools.sssd_conf(domain_section, domain_params)
    start_sssd = 'systemctl restart sssd'
    session_multihost.client[0].run_command(start_sssd)

    def removesssd():
        """ Remove sssd configuration """
        stop_sssd = 'systemctl stop sssd'
        session_multihost.client[0].run_command(stop_sssd)
        removeconf = 'rm -f %s' % (SSSD_DEFAULT_CONF)
        session_multihost.client[0].run_command(removeconf)
    request.addfinalizer(removesssd)


@pytest.fixture(scope="class")
def multihost(session_multihost, request):
    """ Multihost fixture to be used by tests
    :param obj session_multihost: multihost object
    :return obj session_multihost: return multihost object
    """
    if hasattr(request.cls(), 'class_setup'):
        request.cls().class_setup(session_multihost)
        request.addfinalizer(
            lambda: request.cls().class_teardown(session_multihost))
    return session_multihost


@pytest.fixture(scope='class')
def create_posix_usersgroups(session_multihost):
    """ Create posix user and groups """
    ldap_uri = 'ldap://%s' % (session_multihost.master[0].sys_hostname)
    ds_rootdn = 'cn=Directory Manager'
    ds_rootpw = 'Secret123'
    ldap_inst = LdapOperations(ldap_uri, ds_rootdn, ds_rootpw)
    krb = krb5srv(session_multihost.master[0], 'EXAMPLE.TEST')
    for i in range(10):
        user_info = {'cn': 'foo%d' % i,
                     'uid': 'foo%d' % i,
                     'uidNumber': '1458310%d' % i,
                     'gidNumber': '14564100'}
        if ldap_inst.posix_user("ou=People", "dc=example,dc=test", user_info):
            krb.add_principal('foo%d' % i, 'user', 'Secret123')
        else:
            print("Unable to add ldap User %s" % (user_info))
            assert False
    memberdn = 'uid=%s,ou=People,dc=example,dc=test' % ('foo0')
    group_info = {'cn': 'ldapusers',
                  'gidNumber': '14564100',
                  'uniqueMember': memberdn}
    try:
        ldap_inst.posix_group("ou=Groups", "dc=example,dc=test", group_info)
    except LdapException:
        assert False
    group_dn = 'cn=ldapusers,ou=Groups,dc=example,dc=test'
    for i in range(1, 10):
        user_dn = 'uid=foo%d,ou=People,dc=example,dc=test' % i
        add_member = [(ldap.MOD_ADD, 'uniqueMember', user_dn.encode('utf-8'))]
        (ret, _) = ldap_inst.modify_ldap(group_dn, add_member)
        assert ret == 'Success'


@pytest.fixture(scope='class')
def create_posix_usersgroups_failover(session_multihost):
    """ Create posix user and groups """
    for idx in range(2):
        ldap_uri = 'ldap://%s' % (session_multihost.master[idx].sys_hostname)
        ds_rootdn = 'cn=Directory Manager'
        ds_rootpw = 'Secret123'
        ldap_inst = LdapOperations(ldap_uri, ds_rootdn, ds_rootpw)
        for i in range(10):
            user_info = {'cn': 'foo%d' % i,
                         'uid': 'foo%d' % i,
                         'uidNumber': '1458310%d' % i,
                         'gidNumber': '14564100'}
            ldap_inst.posix_user("ou=People", "dc=example,dc=test", user_info)

        memberdn = 'uid=%s,ou=People,dc=example,dc=test' % ('foo0')
        group_info = {'cn': 'ldapusers',
                      'gidNumber': '14564100',
                      'uniqueMember': memberdn}
        try:
            ldap_inst.posix_group("ou=Groups", "dc=example,dc=test",
                                  group_info)
        except LdapException:
            assert False

        group_dn = 'cn=ldapusers,ou=Groups,dc=example,dc=test'
        for i in range(1, 10):
            user_dn = 'uid=foo%d,ou=People,dc=example,dc=test' % i
            add_member = [(ldap.MOD_ADD, 'uniqueMember',
                           user_dn.encode('utf-8'))]
            (ret, _) = ldap_inst.modify_ldap(group_dn, add_member)
            assert ret == 'Success'


@pytest.fixture(scope='class')
def create_posix_usersgroups_autoprivategroups(session_multihost):
    """ Create posix user and groups for autoprivategroup fixture"""
    ldap_uri = 'ldap://%s' % (session_multihost.master[0].sys_hostname)
    ds_rootdn = 'cn=Directory Manager'
    ds_rootpw = 'Secret123'
    ldap_inst = LdapOperations(ldap_uri, ds_rootdn, ds_rootpw)
    for i in range(10):
        user_info = {'cn': 'foobar%d' % i,
                     'uid': 'foobar%d' % i,
                     'uidNumber': '1234560%d' % i,
                     'gidNumber': '1234560%d' % i}
        try:
            ldap_inst.posix_user("ou=People", ds_suffix, user_info)
        except LdapException:
            assert False
    user_info = {'cn': 'foobar11',
                 'uid': 'foobar11',
                 'uidNumber': '14583103',
                 'gidNumber': '14444444'}
    try:
        ldap_inst.posix_user("ou=People", ds_suffix, user_info)
    except LdapException:
        assert False
    memberdn = 'uid=%s,ou=People,dc=example,dc=test' % ('foobar0')
    for i in range(4):
        group_info = {'cn': 'foobar%d' % i,
                      'gidNumber': '1234560%d' % i,
                      'uniqueMember': memberdn}
        try:
            ldap_inst.posix_group("ou=Groups", ds_suffix, group_info)
        except LdapException:
            assert False
    memberdn = 'uid=%s,ou=People,dc=example,dc=test' % ('foobar11')
    group_info = {'cn': 'foobar11',
                  'gidNumber': '14444444',
                  'uniqueMember': memberdn}
    try:
        ldap_inst.posix_group("ou=Groups", ds_suffix, group_info)
    except LdapException:
        assert False
    group_dn = 'cn=foobar0,ou=Groups,%s' % ds_suffix
    user_dn = 'uid=foobar1,ou=People,%s' % ds_suffix
    add_member = [(ldap.MOD_ADD, 'uniqueMember', user_dn.encode('utf-8'))]
    (ret, _) = ldap_inst.modify_ldap(group_dn, add_member)
    assert ret == 'Success'
    group_dn = 'cn=foobar0,ou=Groups,%s' % ds_suffix
    modify_gid = [(ldap.MOD_REPLACE, 'gidNumber', [b'20000'])]
    (ret, return_value) = ldap_inst.modify_ldap(group_dn, modify_gid)
    if not return_value:
        raise LdapException(
            'fail to modify gid, Error:%s' % (ret))
    else:
        print('modified gid')


@pytest.fixture(scope='class')
def netgroups(session_multihost):
    """ Create netgroup users """
    ldap_uri = 'ldap://%s' % (session_multihost.master[0].sys_hostname)
    ds_rootdn = 'cn=Directory Manager'
    ds_rootpw = 'Secret123'
    ldap_inst = LdapOperations(ldap_uri, ds_rootdn, ds_rootpw)
    ldap_inst.org_unit('Netgroups', ds_suffix)
    for idx in range(10):
        nisNetgroupTriple = "(,foo%d, %s)" % (idx, krb_realm)
        netgroup_dn = 'cn=netgroup_%d,ou=Netgroups,%s' % (idx, ds_suffix)
        ldap_inst.create_netgroup(netgroup_dn, nisNetgroupTriple)
    nisNetgroupTriple = "(Host1.example.com,User1,example.com)"
    netgroup_dn = 'cn=NetGroup_CS1,ou=Netgroups,%s' % (ds_suffix)
    ldap_inst.create_netgroup(netgroup_dn, nisNetgroupTriple)
    user = "cn=NetGroup_CS1,ou=Netgroups,dc=example,dc=test"
    add_member = [(ldap.MOD_ADD, 'cn',
                   "NetGroup_CS1_Alias".encode('utf-8'))]
    ldap_inst.modify_ldap(user, add_member)


@pytest.fixture(scope='class')
def write_journalsssd(session_multihost, request):
    """ Creating /etc/sysconfig/sssd and start systemd-journald service"""
    contents = "DEBUG_LOGGER=--logger=journald"
    session_multihost.client[0].put_file_contents('/etc/sysconfig/sssd',
                                                  contents)
    restart_journald = 'systemctl restart systemd-journald'
    session_multihost.client[0].run_command(restart_journald)

    def remove_journalsssd():
        """ Remove  /etc/sysconfig/sssd"""
        cmd = 'rm -f /etc/sysconfig/sssd'
        restart_journald = 'systemctl restart systemd-journald'
        restart_sssd = 'systemctl restart sssd'
        session_multihost.client[0].run_command(cmd)
        session_multihost.client[0].run_command(restart_journald)
        session_multihost.client[0].run_command(restart_sssd)
    request.addfinalizer(remove_journalsssd)


@pytest.fixture(scope="class")
def enable_autofs_schema(session_multihost, request):
    """ Enable autofs schema(rfc2307) to Windows AD.
    :param obj session_multihost: multihost object
    :param obj request: pytest request object
    """
    ldap_uri = 'ldap://%s' % (session_multihost.master[0].sys_hostname)
    ds_rootdn = 'cn=Directory Manager'
    ds_rootpw = 'Secret123'
    ldap_inst = LdapOperations(ldap_uri, ds_rootdn, ds_rootpw)
    ldap_inst.autofs_nis_schema(ds_suffix)


@pytest.fixture(scope="class")
def enable_autofs_service(session_multihost,
                          request):  # pylint: disable=unused-argument
    """ Enable autofs service """
    client = sssdTools(session_multihost.client[0])
    client.backup_sssd_conf()
    services = {'services': 'nss, pam, autofs'}
    client.sssd_conf('sssd', services)
    domain_name = client.get_domain_section_name()
    domain_section = 'domain/{}'.format(domain_name)
    sssd_params = {'autofs_provider': 'ldap',
                   'debug_level': '9'}
    client.sssd_conf(domain_section, sssd_params)

    def disable_autofs_service():
        """ Disable autofs service in sssd.conf """
        services = {'services': 'nss, pam'}
        client.sssd_conf('sssd', services)
        client.clear_sssd_cache()
        stop_autofs = 'systemctl stop autofs'
        session_multihost.client[0].run_command(stop_autofs)
    request.addfinalizer(disable_autofs_service)


@pytest.fixture(scope="class")
def default_sssd(session_multihost, request):
    """ Setup default sssd.conf """
    contents = '''[sssd]
services = nss, pam '''
    session_multihost.client[0].put_file_contents('%s' % (SSSD_DEFAULT_CONF),
                                                  contents)
    tools = sssdTools(session_multihost.client[0])
    tools.fix_sssd_conf_perms()

    def remove_default_sssd():
        """ Remove default sssd.conf """
        session_multihost.client[0].service_sssd('stop')
        cmd = 'rm -f %s' % SSSD_DEFAULT_CONF
        session_multihost.client[0].run_command(cmd)
    request.addfinalizer(remove_default_sssd)


@pytest.fixture(scope='class')
def krb_connection_timeout(
        session_multihost,
        create_host_user,
        request):
    """ Create necessary principals and keytabs for the test.
    :param obj session_multihost: multihost object
    :param obj create_host_user: to create user for SASL with GSSAPI
     authentication
    :param obj request: pytest request object
    """
    krb = krb5srv(session_multihost.master[0], 'EXAMPLE.TEST')

    princ = "ldap/%s" % session_multihost.master[0].sys_hostname
    krb.add_principal(None, None, 'Secret123', 'ldap', '%s@EXAMPLE.TEST' %
                      session_multihost.master[0].sys_hostname)
    keytab = krb.add_remove_keytab(princ, None, 'add')
    if keytab is True:
        assert True
    else:
        assert False
    krb.add_principal(None, None, 'Secret123', 'host', '%s@EXAMPLE.TEST' %
                      session_multihost.client[0].sys_hostname)
    princ = "host/%s" % session_multihost.client[0].sys_hostname
    keytab = krb.add_remove_keytab(princ,
                                   "/opt/sssd_client_valid.keytab", 'add')
    if keytab is True:
        assert True
    else:
        assert False
    chmod_cmd = "chmod 777 /etc/krb5.keytab"
    session_multihost.master[0].run_command(chmod_cmd)
    restore_selinux = "restorecon -v /etc/krb5.keytab"
    session_multihost.master[0].run_command(restore_selinux)
    # Setup client machine
    session_multihost.master[0].transport.get_file(
        "/opt/sssd_client_valid.keytab",
        "/tmp/sssd_client_valid.keytab")
    session_multihost.client[0].transport.put_file(
        "/tmp/sssd_client_valid." "keytab", "/etc/krb5.keytab")
    session_multihost.client[0].run_command(restore_selinux)
    sssd_tools = sssdTools(session_multihost.client[0])
    sssd_tools.remove_sss_cache('/var/lib/sss/db/')
    sssd_tools.fix_sssd_conf_perms()


@pytest.fixture(scope='class')
def create_host_user(session_multihost):
    """ Add host user for SASL with GSSAPI authentication
    :param obj session_multihost: multihost object
    """
    ldap_uri = 'ldap://%s' % (session_multihost.master[0].sys_hostname)
    ds_rootdn = 'cn=Directory Manager'
    ds_rootpw = 'Secret123'
    ldap_inst = LdapOperations(ldap_uri, ds_rootdn, ds_rootpw)
    user_info = {'cn': 'host/%s' % session_multihost.client[0].sys_hostname,
                 'uid': 'host/%s' % session_multihost.client[0].sys_hostname,
                 'uidNumber': '9003',
                 'gidNumber': '9003'}
    ldap_inst.posix_user("ou=People", "dc=example,dc=test", user_info)
    group_dn = 'cn=ldapusers,ou=Groups,dc=example,dc=test'
    uid = 'uid=host/%s' % session_multihost.client[0].sys_hostname
    user_dn = '%s,ou=People,dc=example,dc=test' % uid
    add_member = [(ldap.MOD_ADD, 'uniqueMember', user_dn.encode('utf-8'))]
    (ret, _) = ldap_inst.modify_ldap(group_dn, add_member)
    assert ret == 'Success'


@pytest.fixture(scope='class')
def enable_password_check_syntax(session_multihost, request):
    """ Enable passwordCheckSyntax """
    ldap_uri = 'ldap://%s' % (session_multihost.master[0].sys_hostname)
    ds_rootdn = 'cn=Directory Manager'
    ds_rootpw = 'Secret123'
    ldap_obj = LdapOperations(ldap_uri, ds_rootdn, ds_rootpw)
    mod_dn1 = 'cn=config'
    add_pass_check = [(ldap.MOD_ADD, 'passwordCheckSyntax', [b'on'])]
    (ret, _) = ldap_obj.modify_ldap(mod_dn1, add_pass_check)
    assert ret == 'Success'
    restart_service = 'systemctl restart dirsrv@%s' % ds_instance_name
    session_multihost.master[0].run_command(restart_service)


@pytest.fixture(scope='class')
def enable_ssh_schema(session_multihost, request):
    """ Enable OpenSSH lpk  schema in directory server """
    cwd = os.path.dirname(os.path.abspath(__file__))
    split_cwd = cwd.split('/')
    idx = split_cwd.index('multihost')
    path_list = split_cwd[:idx + 1]
    sssd_qe_path = '/'.join(path_list)
    data_path = "%s/data" % sssd_qe_path
    filename = '98openssh-ldap.ldif'
    schema_path = '/etc/dirsrv/slapd-%s/schema' % ds_instance_name
    remote_file_path = posixpath.join(schema_path, filename)
    source_file_path = posixpath.join(data_path, filename)
    chown = 'chown nobody.nobody %s' % remote_file_path
    chmod = 'chmod 660 %s' % remote_file_path
    session_multihost.master[0].transport.put_file(source_file_path,
                                                   remote_file_path)
    session_multihost.master[0].run_command(chown)
    session_multihost.master[0].run_command(chmod)
    restart_ds = 'systemctl restart dirsrv@%s' % ds_instance_name
    session_multihost.master[0].run_command(restart_ds)


@pytest.fixture(scope='class')
def setup_sshd_authorized_keys(session_multihost, request):
    """ Configuring OpenSSH to Use SSSD for User Keys """
    sshd_conf = '/etc/ssh/sshd_config'
    sshd_conf_bkup = '/etc/ssh/sshd_config.orig'
    bkup_cmd = 'cp %s %s' % (sshd_conf, sshd_conf_bkup)
    session_multihost.client[0].run_command(bkup_cmd)
    session_multihost.client[0].transport.get_file(sshd_conf,
                                                   '/tmp/sshd_config')
    with open('/tmp/sshd_config', 'a+') as conf:
        conf.write('\nAuthorizedKeysCommand /usr/bin/sss_ssh_authorizedkeys')
        conf.write('\nAuthorizedKeysCommandUser nobody\n')
    session_multihost.client[0].transport.put_file('/tmp/sshd_config',
                                                   sshd_conf)
    restart_sshd = 'systemctl restart sshd'
    session_multihost.client[0].run_command(restart_sshd)

    def restore_sshd():
        """ Restore sshd configuration """
        restore_cmd = 'cp %s %s' % (sshd_conf_bkup, sshd_conf)
        session_multihost.client[0].run_command(restore_cmd)
    request.addfinalizer(restore_sshd)


@pytest.fixture(scope='class')
def enable_ssh_responder(session_multihost, request):
    """ Enable ssh responder in sssd.conf """
    tools = sssdTools(session_multihost.client[0])
    tools.backup_sssd_conf()
    session_multihost.client[0].service_sssd('stop')
    tools.remove_sss_cache('/var/lib/sss/db')
    sssd_params = {'services': 'nss, pam, ssh'}
    tools.sssd_conf("sssd", sssd_params, action='update')
    session_multihost.client[0].service_sssd('start')

    def restore_sssd():
        """ Restore sssd.conf """
        tools.restore_sssd_conf()
    request.addfinalizer(restore_sssd)


@pytest.fixture(scope='class')
def enable_sssd_hostmap(session_multihost, request):
    """ Enables sssd for network and host database in nsswitch.conf """
    tools = sssdTools(session_multihost.client[0])
    # Since Fedora 36+ support of user-nsswitch was dropped
    # This applies to CentOS 10 and RHEL 10
    nsswitch_file = '/etc/authselect/user-nsswitch.conf'
    cmd = session_multihost.client[0].run_command(
        f"test -f {nsswitch_file}", raiseonerr=False)
    has_user_nsswith = cmd.returncode == 0
    if not has_user_nsswith:
        nsswitch_file = "/etc/nsswitch.conf"

    bkup = f'cp -vf {nsswitch_file} {nsswitch_file}_bkp'
    session_multihost.client[0].run_command(bkup)

    for value in ['hosts', 'networks']:
        update_nsswitch = f"sed -i 's/{value}:/{value}: sss/' {nsswitch_file}"
        session_multihost.client[0].run_command(update_nsswitch)
    if has_user_nsswith:
        authselect = 'authselect select sssd'
        session_multihost.client[0].run_command(authselect)

    def restore_nsswitch():
        restore = f"cp -vf {nsswitch_file}_bkp {nsswitch_file}"
        session_multihost.client[0].run_command(restore)
        if has_user_nsswith:
            session_multihost.client[0].run_command(authselect)
        tools.clear_sssd_cache()
        # remove the backup file
        remove_bkup = 'rm -f %s_bkup' % nsswitch_file
        session_multihost.client[0].run_command(remove_bkup)
    request.addfinalizer(restore_nsswitch)


@pytest.fixture(scope='class')
def add_host_entry(session_multihost, request):
    """
    Add host and network entries in Directory server to be used
    by sssd hostmap feature.
    """
    ldap_uri = 'ldap://%s' % (session_multihost.master[0].sys_hostname)
    ds_rootdn = 'cn=Directory Manager'
    ds_rootpw = 'Secret123'
    ldap_inst = LdapOperations(ldap_uri, ds_rootdn, ds_rootpw)
    user_info = {'cn': 'node1'.encode('utf-8'),
                 'objectClass': [b'top', b'ipHost', b'device'],
                 'ipHostNumber': '192.168.1.1'.encode('utf-8')}
    user_dn = 'cn=node1,ou=People,dc=example,dc=test'
    (_, _) = ldap_inst.add_entry(user_info, user_dn)
    user_info = {'cn': 'node2'.encode('utf-8'),
                 'objectClass': [b'top', b'ipNetwork', b'device'],
                 'ipNetworkNumber': '192.168.1.2'.encode('utf-8')}
    user_dn = 'cn=node2,ou=People,dc=example,dc=test'
    (_, _) = ldap_inst.add_entry(user_info, user_dn)
    user_info = {'cn': 'node3'.encode('utf-8'),
                 'objectClass': [b'top', b'ipHost', b'device', b'ipNetwork'],
                 'ipNetworkNumber': '192.168.1.3'.encode('utf-8'),
                 'ipHostNumber': '192.168.1.3'.encode('utf-8')}
    user_dn = 'cn=node3,ou=People,dc=example,dc=test'
    (_, _) = ldap_inst.add_entry(user_info, user_dn)

    def remove_host_entry():
        """ Removing Hostmap entries from ldap """
        for node in ['node1', 'node2', 'node3']:
            dn = f'cn={node},ou=People,dc=example,dc=test'
            print("deleting %s" % dn)
            (_, _) = ldap_inst.del_dn(dn)
    request.addfinalizer(remove_host_entry)


def execute_cmd(session_multihost, command):
    """ Execute command on client """
    cmd = session_multihost.client[0].run_command(command)
    return cmd


@pytest.fixture(scope='class')
def ns_account_lock(session_multihost, request):
    """ Backup and restore sssd.conf """
    version = float(re.findall(r"\d+\.\d+",
                               session_multihost.client[0].distro)[0])
    if version >= 9:
        execute_cmd(session_multihost, "yum install -y 389-ds-base")
    else:
        execute_cmd(session_multihost, "yum module "
                                       "enable -y 389-ds; "
                                       "yum install -y "
                                       "389-ds-base")
    tools = sssdTools(session_multihost.client[0])
    domain_name = tools.get_domain_section_name()
    client = sssdTools(session_multihost.client[0])
    domain_params = {'cache_credentials': 'true',
                     'enumerate': 'true',
                     'access_provider': 'ldap',
                     'ldap_access_order': 'expire',
                     'ldap_account_expire_policy': '389DS',
                     'ldap_ns_account_lock': 'nsAccountlock'}
    client.sssd_conf(f'domain/{domain_name}', domain_params)
    domain_params = {'filter_groups': 'root',
                     'filter_users': 'root',
                     'debug_level': '9'}
    client.sssd_conf('nss', domain_params)
    session_multihost.client[0].service_sssd('restart')
    # Add managed role
    master_e = session_multihost.master[0].ip
    ldap_uri = f'ldap://{master_e}'
    ds_rootdn = 'cn=Directory Manager'
    ds_rootpw = 'Secret123'
    ldap_inst = LdapOperations(ldap_uri, ds_rootdn, ds_rootpw)
    user_info = {'cn': 'managed'.encode('utf-8'),
                 'objectClass': [b'top', b'LdapSubEntry',
                                 b'nsRoleDefinition',
                                 b'nsSimpleRoleDefinition',
                                 b'nsManagedRoleDefinition']}
    user_dn = 'cn=managed,ou=People,dc=example,dc=test'
    (_, _) = ldap_inst.add_entry(user_info, user_dn)
    user_dn = 'uid=foo1,ou=People,dc=example,dc=test'
    role_dn = "cn=managed,ou=people,dc=example,dc=test"
    add_member = [(ldap.MOD_ADD, 'nsRoleDN', role_dn.encode('utf-8'))]
    (ret, _) = ldap_inst.modify_ldap(user_dn, add_member)
    assert ret == 'Success'
    # Add filter role
    user_info = {'cn': 'filtered'.encode('utf-8'),
                 'objectClass': [b'top', b'LdapSubEntry',
                                 b'nsRoleDefinition',
                                 b'nsComplexRoleDefinition',
                                 b'nsFilteredRoleDefinition'],
                 'nsRoleFilter': 'o=filtered'.encode('utf-8'),
                 'Description': 'filtered role'.encode('utf-8')}
    user_dn = 'cn=filtered,ou=People,dc=example,dc=test'
    (_, _) = ldap_inst.add_entry(user_info, user_dn)
    user_dn = 'uid=foo4,ou=People,dc=example,dc=test'
    role_dn = "filtered"
    add_member = [(ldap.MOD_ADD, 'o', role_dn.encode('utf-8'))]
    (ret, _) = ldap_inst.modify_ldap(user_dn, add_member)
    assert ret == 'Success'

    def restoresssdconf():
        """ Restore sssd.conf """
        master_e = session_multihost.master[0].ip
        ldap_uri = f'ldap://{master_e}'
        ds_rootdn = 'cn=Directory Manager'
        ds_rootpw = 'Secret123'
        ldap_inst = LdapOperations(ldap_uri, ds_rootdn, ds_rootpw)
        user_dn = 'uid=foo1,ou=People,dc=example,dc=test'
        role_dn = "cn=managed,ou=people,dc=example,dc=test"
        del_member = [(ldap.MOD_DELETE, 'nsRoleDN', role_dn.encode('utf-8'))]
        (ret, _) = ldap_inst.modify_ldap(user_dn, del_member)
        assert ret == 'Success'
        user_dn = 'uid=foo4,ou=People,dc=example,dc=test'
        role_dn = "filtered"
        del_member = [(ldap.MOD_DELETE, 'o', role_dn.encode('utf-8'))]
        (ret, _) = ldap_inst.modify_ldap(user_dn, del_member)
        assert ret == 'Success'
        for i in ['cn=managed,ou=people,dc=example,dc=test',
                  'cn=filtered,ou=people,dc=example,dc=test',
                  'cn=nested,ou=People,dc=example,dc=test']:
            ldap_inst.del_dn(i)
    request.addfinalizer(restoresssdconf)

# ====================  Session Scoped Fixtures ================


@pytest.fixture(scope="session", autouse=True)
# pylint: disable=unused-argument
def setup_session(session_multihost, request, create_testdir):
    """
    Session fixture which calls fixture in order before tests run
    :param obj session_multihost: multihost object
    :param obj request: pytest request object
    """
    client_libs = sssdTools(session_multihost.client[0])
    client_libs.client_install_pkgs()
    client_libs.authselect()
    restart_rsyslog = 'systemctl restart rsyslog'
    session_multihost.client[0].run_command(restart_rsyslog)
    for idx in range(2):
        master_libs = sssdTools(session_multihost.master[idx])
        master_libs.server_install_pkgs()
        master_libs.authselect()

    def teardown_session():
        """ Teardown session """
        print("i am in teardown session")
    request.addfinalizer(teardown_session)



""" Automation of Krb5 tests

:subsystemteam: sst_idm_sssd
:upstream: yes
:status: approved
"""
from __future__ import print_function
import pytest
from sssd.testlib.common.utils import sssdTools
from sssd.testlib.common.ssh2_python import check_login_client, check_login_client_bool


@pytest.mark.usefixtures('setup_sssd_krb', 'create_posix_usersgroups')
@pytest.mark.krb5
class TestKrbWithLogin(object):
    @pytest.mark.tier1
    def test_0001_krb5_not_working_based_on_k5login(self,
                                                    multihost,
                                                    localusers,
                                                    backupsssdconf):
        """
        :title: krb5: access_provider = krb5 is not
         working in RHEL8 while restricting logins
         based on .k5login file
        :id: dfc177ff-58a7-4697-8d23-e444928c7092
        :casecomponent: authselect
        :customerscenario: True
        :requirement: IDM-SSSD-REQ :: Authselect replaced authconfig
        :bugzilla:
         https://bugzilla.redhat.com/show_bug.cgi?id=1734094
        """
        multihost.client[0].run_command('authselect select sssd '
                                        'with-files-access-provider')
        multihost.client[0].service_sssd('stop')
        client_tool = sssdTools(multihost.client[0])
        domain_params = {'proxy_lib_name': 'files',
                         'passwd_files': '/etc/passwd',
                         'proxy_pam_target': 'sssd-shadowutils',
                         'access_provider': 'krb5'}
        client_tool.sssd_conf('domain/example1', domain_params)
        dmain_delete = {"ldap_user_home_directory": "/home/%u",
                        "ldap_uri": multihost.master[0].sys_hostname,
                        "ldap_search_base": "dc=example,dc=test",
                        "ldap_tls_cacert": "/etc/openldap/cacerts/cacert.pem",
                        "use_fully_qualified_names": "True"}
        client_tool.sssd_conf('domain/example1', dmain_delete, action='delete')
        multihost.client[0].service_sssd('start')
        user = 'user5000'
        multihost.client[0].run_command(f'touch /home/{user}/.k5login')
        multihost.client[0].run_command(f'chown {user} /home/{user}/.k5login')
        multihost.client[0].run_command(f'chgrp {user} /home/{user}/.k5login')
        multihost.client[0].run_command(f'chmod 664 /home/{user}/.k5login')
        multihost.client[0].service_sssd('restart')
        ssh = check_login_client_bool(multihost, user, "Secret123")
        multihost.client[0].run_command(f'rm -vf /home/{user}/.k5login')
        multihost.client[0].service_sssd('restart')
        assert ssh, f"{user} is not able to login"
        check_login_client(multihost, user, 'Secret123')
        multihost.client[0].run_command('authselect select sssd')

    @pytest.mark.tier1_2
    def test_0002_generating_lot_of(self, multihost, backupsssdconf):
        """
        :title: SSSD is generating lot of LDAP
         queries in a very large environment
        :bugzilla: https://bugzilla.redhat.com/show_bug.cgi?id=1772513
        :id: 74a60320-e48b-11eb-ba19-845cf3eff344
        :requirement: IDM-SSSD-REQ : LDAP Provider
        :steps:
          1. Start SSSD with any configuration
          2. Call 'getent passwd username@domain'
          3. Check the entry is present in data and timestamp cache
          4. Now stop SSSD and remove the timestamp cache
          5. Start SSSD and call 'sss_cache -E'
          6. Call 'getent passwd username@domain'
          7. Do the ldbsearch checks again
        :expectedresults:
          1. Should succeed
          2. Should succeed
          3. Should succeed
          4. Should succeed
          5. Should succeed
          6. Should succeed
          7. Should succeed
        """
        multihost.client[0].service_sssd('restart')
        cmd = multihost.client[0].run_command('getent passwd '
                                              'foo1@example1')
        assert 'foo1@example1' in cmd.stdout_text
        multihost.client[0].run_command("yum install -y ldb-tools")
        sssd_client = multihost.client[0]
        cmd_search1 = sssd_client.run_command("ldbsearch "
                                              "-H /var/lib/sss/db/"
                                              "cache_example1.ldb  -b "
                                              "name=foo1@example1,"
                                              "cn=users,cn=example1,"
                                              "cn=sysdb")
        assert 'name=foo1@example1,cn=users,cn=example1,cn=sysdb' \
               in cmd_search1.stdout_text
        cmd_search2 = sssd_client.run_command('ldbsearch -H '
                                              '/var/lib/sss/db/'
                                              'timestamps_example1.ldb '
                                              '-b name=foo1@example1,'
                                              'cn=users,cn=example1,'
                                              'cn=sysdb')
        assert "dn: name=foo1@example1,cn=users,cn=example1,cn=sysdb" in \
               cmd_search2.stdout_text
        multihost.client[0].run_command("rm -vf /var/lib/sss/db/"
                                        "timestamps_example1.ldb")
        multihost.client[0].service_sssd('restart')
        multihost.client[0].run_command("sss_cache -E")
        cmd = multihost.client[0].run_command('getent passwd '
                                              'foo1@example1')
        assert 'foo1@example1' in cmd.stdout_text
        cmd_search1 = multihost.client[0].run_command(
            "ldbsearch -H /var/lib/sss/db/cache_example1.ldb -b "
            "name=foo1@example1,cn=users,cn=example1,cn=sysdb")
        assert 'name=foo1@example1,cn=users,cn=example1,cn=sysdb' in \
               cmd_search1.stdout_text
        cmd_search2 = multihost.client[0].run_command(
            'ldbsearch -H /var/lib/sss/db/timestamps_example1.ldb -b '
            'name=foo1@example1,cn=users,cn=example1,cn=sysdb')
        assert "dn: name=foo1@example1,cn=users,cn=example1,cn=sysdb" \
               in cmd_search2.stdout_text




""" Common AD Fixtures """
from __future__ import print_function
import subprocess
import random
import pytest
from sssd.testlib.common.paths import SSSD_DEFAULT_CONF
from sssd.testlib.common.utils import sssdTools


pytest_plugins = (
    'sssd.testlib.common.fixtures',
    'pytest_importance',
    'pytest_ticket',
    'sssd.testlib.common.custom_log',
)


def pytest_configure():
    """ Namespace hook, Adds below dict to pytest namespace """
    pytest.num_masters = 0
    pytest.num_ad = 4
    pytest.num_atomic = 0
    pytest.num_replicas = 0
    pytest.num_clients = 1
    pytest.num_others = 0


# ######## Function scoped Fixtures ####################
@pytest.fixture(scope="function")
def newhostname(session_multihost, request):
    """ Change client hostname to a truncated version in the AD domain"""
    cmd = session_multihost.client[0].run_command(
        'hostname', raiseonerr=False)
    ad_domain = session_multihost.ad[0].domainname
    old_hostname = cmd.stdout_text.rstrip()
    hostname = f'client{random.randint(1,99)}.{ad_domain}'
    session_multihost.client[0].run_command(
        f'hostname {hostname}', raiseonerr=False)

    def restore():
        """ Restore hostname """
        session_multihost.client[0].run_command(
            f'hostname {old_hostname}',
            raiseonerr=False
        )
    request.addfinalizer(restore)


@pytest.fixture(scope="function")
def adjoin(session_multihost, request):
    """ Join to AD using net ads command """
    ad_realm = session_multihost.ad[0].realm
    ad_ip = session_multihost.ad[0].ip
    ad_host = session_multihost.ad[0].sys_hostname
    client_ad = sssdTools(session_multihost.client[0], session_multihost.ad[0])
    client_ad.update_resolv_conf(session_multihost.ad[0])
    client_ad.update_resolv_conf(session_multihost.ad[len(session_multihost.ad)-1])
    client_ad.systemsssdauth(ad_realm, ad_host)
    client_ad.disjoin_ad(raiseonerr=False)  # Make sure system is disjoined from AD
    client_ad.create_kdcinfo(ad_realm, ad_ip)
    kinit = f'kinit Administrator@{ad_realm}'
    ad_password = session_multihost.ad[0].ssh_password
    try:
        session_multihost.client[0].run_command(kinit, stdin_text=ad_password)
    except subprocess.CalledProcessError:
        pytest.fail("kinit failed")

    def _join(membersw=None):
        """ Join AD """
        if membersw == 'samba':
            client_ad.join_ad(ad_realm, ad_password, mem_sw='samba')
        else:
            client_ad.join_ad(ad_realm, ad_password)

    def adleave():
        """ Disjoin AD """
        client_ad.disjoin_ad(raiseonerr=False)
        remove_keytab = 'rm -f /etc/krb5.keytab'
        kdestroy_cmd = 'kdestroy -A'
        session_multihost.client[0].run_command(kdestroy_cmd)
        session_multihost.client[0].run_command(remove_keytab)
    request.addfinalizer(adleave)
    return _join


@pytest.fixture(autouse=True)
def capture_sssd_logs(session_multihost, request):
    """This will print sssd logs in case of test failure"""
    yield
    if request.session.testsfailed:
        client = session_multihost.client[0]
        print(f"\n\n===Logs for {request.node.name}===\n\n")
        for data_d in client.run_command("ls /var/log/sssd/").stdout_text.split():
            client.run_command(f'echo "--- {data_d} ---"; '
                               f'cat /var/log/sssd/{data_d}')


@pytest.fixture(scope="function")
def adchildjoin(session_multihost, request):
    """ Join to AD using net ads command """
    ad_realm = session_multihost.ad[1].realm
    ad_ip = session_multihost.ad[1].ip
    client_ad = sssdTools(session_multihost.client[0], session_multihost.ad[1])
    client_ad.disjoin_ad(raiseonerr=False)
    client_ad.create_kdcinfo(ad_realm, ad_ip)
    kinit = "kinit Administrator@%s" % ad_realm
    ad_password = session_multihost.ad[1].ssh_password
    try:
        session_multihost.client[0].run_command(kinit, stdin_text=ad_password)
    except subprocess.CalledProcessError:
        pytest.fail("kinit failed")

    def _join(membersw=None):
        """ Join AD """
        if membersw == 'samba':
            client_ad.join_ad(ad_realm, ad_password, mem_sw='samba')
        else:
            client_ad.join_ad(ad_realm, ad_password)

    def adleave():
        """ Disjoin AD """
        client_ad.disjoin_ad(raiseonerr=False)
        remove_keytab = 'rm -f /etc/krb5.keytab'
        kdestroy_cmd = 'kdestroy -A'
        session_multihost.client[0].run_command(kdestroy_cmd)
        session_multihost.client[0].run_command(remove_keytab)
    request.addfinalizer(adleave)
    return _join


@pytest.fixture(scope='function')
def backupsssdconf(session_multihost, request):
    """ Backup and restore sssd.conf """
    bkup = 'cp -f %s %s.orig' % (SSSD_DEFAULT_CONF,
                                 SSSD_DEFAULT_CONF)
    session_multihost.client[0].run_command(bkup)
    session_multihost.client[0].service_sssd('stop')

    def restoresssdconf():
        """ Restore sssd.conf """
        restore = 'cp -f %s.orig %s' % (SSSD_DEFAULT_CONF, SSSD_DEFAULT_CONF)
        session_multihost.client[0].run_command(restore)
    request.addfinalizer(restoresssdconf)


# ############## class scoped Fixtures ##############################
@pytest.fixture(scope="class")
def multihost(session_multihost, request):
    """ Multihost fixture to be used by tests
    :param obj session_multihost: multihost object
    :return obj session_multihost: return multihost object
    :Exceptions: None
    """
    if hasattr(request.cls(), 'class_setup'):
        request.cls().class_setup(session_multihost)
        request.addfinalizer(
            lambda: request.cls().class_teardown(session_multihost))
    return session_multihost


@pytest.fixture(scope="class")
def clear_sssd_cache(session_multihost):
    """ Clear sssd cache """
    client = sssdTools(session_multihost.client[0])
    client.clear_sssd_cache()


# ################### Session scoped fixtures #########################
@pytest.fixture(scope="session", autouse=True)
def setup_session(request, session_multihost, create_testdir):
    """ Setup Session """
    client = sssdTools(session_multihost.client[0])
    realm = session_multihost.ad[1].realm
    ad_host = session_multihost.ad[1].sys_hostname
    try:
        master = sssdTools(session_multihost.master[0])
    except IndexError:
        pass
    else:
        master.server_install_pkgs()
        master.update_resolv_conf(session_multihost.ad[1].ip)
    client.client_install_pkgs()
    client.update_resolv_conf(session_multihost.ad[1].ip)
    # Sssd is not configured yet so we do not start it here.
    client.clear_sssd_cache(start=False)
    client.systemsssdauth(realm, ad_host)

    def teardown_session():
        """ Teardown session """
        session_multihost.client[0].service_sssd('stop')
        remove_sssd_conf = 'rm -f /etc/sssd/sssd.conf'
        session_multihost.client[0].run_command(remove_sssd_conf)
    request.addfinalizer(teardown_session)



""" AD Sudo test cases

:requirement: ad_sudo
:casecomponent: sssd
:subsystemteam: sst_idm_sssd
:upstream: yes
:status: approved
"""

import re
import pytest
from sssd.testlib.common.utils import sssdTools


@pytest.mark.usefixtures('enable_ad_sudoschema',
                         'create_ad_sudousers',
                         'sudorules',
                         'joinad')
@pytest.mark.sudo
@pytest.mark.tier2
class TestSudo(object):
    """ Automation of BZ's related to Sudo with AD """

    @classmethod
    def class_setup(cls, multihost):
        """ Set sudo provider to AD """
        client = sssdTools(multihost.client[0], multihost.ad[0])
        client.sssd_conf('sssd', {'services': 'nss, pam, sudo'})
        params = {
            'use_fully_qualified_names': 'False',
            'sudo_provider': 'ad',
            'debug_level': '9'}
        client.sssd_conf(f'domain/{client.get_domain_section_name()}', params)
        enable_sssd = 'authselect select sssd with-sudo with-mkhomedir --force'
        multihost.client[0].run_command(enable_sssd, raiseonerr=False)
        client.clear_sssd_cache()

    @staticmethod
    @pytest.mark.converted('test_sudo.py', 'test_sudo__case_sensitive_false')
    def test_001_bz1380436(multihost):
        """test_001_bz1380436

        :title: IDM-SSSD-TC: ad_provider: ad_sudo: ignore case
         on case insenstive Domain
        :id: 6cc67f37-808a-4b2c-a2cc-e4e4812388f4
        :bugzilla:
          https://bugzilla.redhat.com/show_bug.cgi?id=1622109
          https://bugzilla.redhat.com/show_bug.cgi?id=1519287
        :customerscenario: True
        :steps:
          1. Add sudo rules containing upper and lower case user names
          2. Login with lower and upper user case names
        :expectedresults:
          1. Should succeed
          2. Verify the the user when logged in with upper
             and lower case can fetch the sudo rules from AD
        :description: Sudo rules used in the fixture contain
          multiple sudoUser attributes.
        """
        realm = multihost.ad[0].realm
        adusers = ['sudo_idmuser1', 'SUDO_IDMUSER1', 'sudo_idmuser2',
                   f'sudo_idmuser3@{realm}', 'sudo_idmuser3', 'SUDO_IDMUSER3']
        failed = []
        for user in adusers:
            cmd = multihost.client[0].run_command(
                f'su - {user} -c "sudo -l"', raiseonerr=False)
            if cmd.returncode != 0 or \
                    '(root) NOPASSWD: /usr/bin/less' not in cmd.stdout_text:
                failed.append(user)

        assert not failed, f"Rules missing for users: {','.join(failed)}"

    @staticmethod
    @pytest.mark.converted('test_sudo.py', 'test_sudo__sudo_user_is_group')
    def test_002_bz1372440(multihost):
        """test_002_bz1372440

        :title: IDM-SSSD-TC: ad_provider: ad_sudo: AD managed sudo groups
         will not work with sssd
        :id: 56616411-d56a-4e3f-b732-a39a4fae8bbc
        :bugzilla: https://bugzilla.redhat.com/show_bug.cgi?id=1372440
        :steps:
          1. Add sudo rules with sudoUser attribute set to group names
             (%sudo_idmgroup1 which has member sudo_idmuser1)
          2. Add users to the group.
          3. Verify sudo_idmuser1 can fetch the sudo rule
          4. Run the required command as sudo
        :expectedresults:
          1. Should succeed
          2. Should succeed
          3. Should succeed
          4. Should succeed
        """
        user = 'sudo_idmuser1'

        # Test ssh login
        client = sssdTools(multihost.client[0], multihost.ad[0])
        ssh_result = client.auth_from_client(user, 'Secret123') == 3

        cmd = multihost.client[0].run_command(
            f'su - {user} -c "sudo -l"', raiseonerr=False)

        rule_result = cmd.returncode == 0 and \
            '(root) NOPASSWD: /usr/bin/less' in cmd.stdout_text

        cmd2 = multihost.client[0].run_command(
            f'su - {user} -c "sudo -n /usr/bin/less /etc/passwd"',
            raiseonerr=False)

        assert ssh_result, f"Ssh failed for user: {user}."
        assert rule_result, f"Rules missing for user: {user}."
        assert cmd2.returncode == 0, f"Sudo command failed for user: {user}!"

    @staticmethod
    @pytest.mark.converted('test_sudo.py', 'test_sudo__sudo_user_is_nonposix_group')
    def test_003_support_non_posix_group_in_sudorule(multihost):
        """test_003_support_non_posix_group_in_sudorule

        :title: IDM-SSSD-TC: ad_provider: ad_sudo: support non-posix
         groups in sudo rules
        :id: b2def0eb-772d-41b4-b496-f7b3cb61169d
        :customerscenario: True
        :bugzilla: https://bugzilla.redhat.com/show_bug.cgi?id=1826272
        :steps:
          1. Disable ldap_id_mapping on client
          2. Create a non-posix group in the AD and add an AD-user as a
             members to it
          3. Add a sudo rule in the /etc/sudoers file for this user with
             '%:<group_name>'
          4. List the sudo commands allowed for the user
        :expectedresults:
          1. Should succeed
          2. Should succeed
          3. Should succeed
          4. Verify sudo_userx can fetch the sudo rule and run
             the required command as sudo
        """
        client = sssdTools(multihost.client[0], multihost.ad[0])
        domain_sect = f'domain/{multihost.ad[0].domainname}'
        user = 'sudo_user1'
        client.sssd_conf(domain_sect, {'ldap_id_mapping': 'false'})
        client.clear_sssd_cache()

        # Test ssh login
        client = sssdTools(multihost.client[0], multihost.ad[0])
        ssh_result = client.auth_from_client(user, 'Secret123') == 3

        cmd = multihost.client[0].run_command(
            f'su - {user} -c "sudo -l"', raiseonerr=False)

        test_result = cmd.returncode == 0
        test_result = test_result and '(root) NOPASSWD: /usr/bin/head' in\
            cmd.stdout_text

        client.sssd_conf(
            domain_sect, {'ldap_id_mapping': 'false'}, action='delete')
        client.clear_sssd_cache()
        assert ssh_result, f"Ssh failed for user: {user}."
        assert test_result, f"Rules missing for user: {user}."

    @staticmethod
    @pytest.mark.converted('test_sudo.py', 'test_sudo__runasuser_shortname')
    def test_004_sudorule_with_short_username(multihost):
        """test_004_sudorule_with_short_username

        :title: sssd should accept a short-username to sudoRunAs option
        :id:61b1abf2-310b-4cdf-8238-b32d235df9a9
        :customerscenario: True
        :bugzilla: https://bugzilla.redhat.com/show_bug.cgi?id=1910131
        :setup:
         1. Add sudo rules with sudoRunAs attribute value set to short-username
         2. Join a client, without fqdn, to the AD
         3. Set debug level to 2
        :steps:
          1. Run sudo command as AD-user for whom rule is created
        :expectedResuls:
          1. There should be no error in the sudo or domain log related
             to 'short-username or non-fqdn username'
        """
        client = sssdTools(multihost.client[0], multihost.ad[0])
        domain_name = client.get_domain_section_name()
        domain_section = f'domain/{domain_name}'
        params = {'debug_level': '2'}
        client.sssd_conf(domain_section, params)
        client.sssd_conf('sudo', params)
        multihost.client[0].service_sssd('restart')
        user = 'sudo_usera'
        client.clear_sssd_cache()
        sudo_log = '/var/log/sssd/sssd_sudo.log'
        domain_log = '/var/log/sssd/sssd_%s.log' % domain_name
        for file in sudo_log, domain_log:
            log = multihost.client[0].get_file_contents(file).decode('utf-8')
            msg = 'Unable to parse name (.*) The internal name format '\
                  'cannot be parsed'
            find = re.compile(r'%s' % msg)
            assert not find.search(log)

        # Test ssh login
        client = sssdTools(multihost.client[0], multihost.ad[0])
        ssh_result = client.auth_from_client(user, 'Secret123') == 3

        cmd = multihost.client[0].run_command(
            f'su - {user} -c "sudo -l"', raiseonerr=False)

        test_result = cmd.returncode == 0
        test_result = \
            test_result and '(sudo_idmuser1) NOPASSWD: /usr/bin/head' in\
            cmd.stdout_text

        client.sssd_conf('sudo', params, action='delete')

        assert ssh_result, f"Ssh failed for user: {user}."
        assert test_result, f"Rules missing for user: {user}."

    @classmethod
    def class_teardown(cls, multihost):
        """ Remove sudo provider from Domain section """
        client = sssdTools(multihost.client[0], multihost.ad[0])
        domain_section = f'domain/{client.get_domain_section_name()}'
        client.sssd_conf('sssd', {'services': 'nss, pam'})
        params = {
            'use_fully_qualified_names': 'False',
            'sudo_provider': 'ad',
            'debug_level': '9'}
        client.sssd_conf(domain_section, params, action='delete')

        # Cleanup homes
        multihost.client[0].run_command(
            'rm -rf /home/{sudo,SUDO}_*', raiseonerr=False)



""" AD-Provider AD Login Attributes tests ported from bash

:requirement: ad_login_attr
:casecomponent: sssd
:subsystemteam: sst_idm_sssd
:upstream: yes
:caseautomation: Automated
:testtype: functional
"""


import pytest
from pexpect import pxssh
from sssd.testlib.common.utils import sssdTools


@pytest.fixture(scope="function", name="sssd_domain")
def fixture_sssd_domain(session_multihost, request):
    """Change domain to sssd_domain"""
    session_multihost.client[0].run_command(
        'cp -f /etc/sssd/sssd.conf /etc/sssd/sssd.conf.sssd_domain',
        raiseonerr=False
    )
    client = sssdTools(session_multihost.client[0], session_multihost.ad[0])
    dom_section = f'domain/{client.get_domain_section_name()}'
    session_multihost.client[0].run_command(
        f'sed -i  "s|{dom_section}|domain/sssd_domain|g" /etc/sssd/sssd.conf',
        raiseonerr=False
    )
    client.sssd_conf('sssd', {'domains': 'sssd_domain', })

    def remove_sssd_domain():
        """ Remove sssd_domain"""
        session_multihost.client[0].run_command(
            'mv -f /etc/sssd/sssd.conf.sssd_domain /etc/sssd/sssd.conf',
            raiseonerr=False
        )
    request.addfinalizer(remove_sssd_domain)


@pytest.mark.tier1_3
@pytest.mark.adloginattr
@pytest.mark.usefixtures("joinad")
class TestADLoginAttributes:
    """Automated Test Cases for AD Login Attributes ported from bash"""

    @staticmethod
    def test_0001_login_by_samaccountname(multihost, create_aduser_group):
        """test_0001_login_by_samaccountname

        :title: IDM-SSSD-TC: ad_provider: ad_login_attr: Default behaviour
          by login with sAMAccountName
        :id: ba681368-eefe-459f-906e-1b32d405d496
        :setup:
          1. Configure sssd with ldap_user_principal = userPrincipalName
        :steps:
          1. Run getent passwd for the user.
          2. Run su for the user with short name (without domain).
          3. Run ssh for the user with short name (without domain).
        :expectedresults:
          1. User is found.
          2. Su command succeeds.
          3. Ssh command succeeds.
        :customerscenario: False
        """
        # Configure sssd
        multihost.client[0].service_sssd('stop')
        client = sssdTools(multihost.client[0], multihost.ad[0])
        client.backup_sssd_conf()
        dom_section = f'domain/{client.get_domain_section_name()}'
        sssd_params = {
            'ldap_id_mapping': 'True',
            'use_fully_qualified_names': 'False',
            'debug_level': '9',
            'id_provider': 'ad',
            'ad_domain': multihost.ad[0].domainname.lower(),
            'ad_server': multihost.ad[0].hostname,
            'ldap_schema': 'ad',
            'ldap_user_principal': 'userPrincipalName',
        }
        client.sssd_conf(dom_section, sssd_params)
        client.sssd_conf('sssd', {'services': 'nss, pam, ssh'})
        client.clear_sssd_cache()

        # Create AD user
        (aduser, _) = create_aduser_group

        # Search for the user
        usr_cmd = multihost.client[0].run_command(
            f'getent passwd {aduser}',
            raiseonerr=False
        )

        # Run su command
        su_result = client.su_success(aduser)
        # Run ssh command
        ssh_result = client.auth_from_client(aduser, 'Secret123') == 3

        client.restore_sssd_conf()
        client.clear_sssd_cache()

        # Evaluate test results
        assert usr_cmd.returncode == 0, f"User {aduser} was not found."
        assert su_result, "The su command failed!"
        assert ssh_result, "The ssh login failed!"

    @staticmethod
    def test_0002_login_by_userprincipalname(multihost, create_aduser_group):
        """test_0002_login_by_userprincipalname

        :title: IDM-SSSD-TC: ad_provider: ad_login_attr: Verify user login
         with userPrincipalName
        :id: 589f4fb3-ef62-42c1-827a-77d73129b879
        :setup:
          1. Configure sssd with ldap_user_principal = userPrincipalName
        :steps:
          1. Run su for the user with fully qualified name.
          2. Run ssh for the user with fully qualified name.
        :expectedresults:
          1. Su command succeeds.
          3. Ssh command succeeds.
        :customerscenario: False
        """
        # Configure sssd
        multihost.client[0].service_sssd('stop')
        ad_realm = multihost.ad[0].domainname.upper()
        client = sssdTools(multihost.client[0], multihost.ad[0])
        client.backup_sssd_conf()
        dom_section = f'domain/{client.get_domain_section_name()}'
        sssd_params = {
            'ldap_id_mapping': 'True',
            'debug_level': '9',
            'id_provider': 'ad',
            'use_fully_qualified_names': 'False',
            'ad_domain': multihost.ad[0].domainname.lower(),
            'ad_server': multihost.ad[0].hostname,
            'ldap_schema': 'ad',
            'ldap_user_principal': 'userPrincipalName',
        }
        client.sssd_conf(dom_section, sssd_params)
        client.sssd_conf('sssd', {'services': 'nss, pam, ssh'})
        client.clear_sssd_cache()

        # Create AD user
        (aduser, _) = create_aduser_group

        # Run su command
        su_result = client.su_success(f'{aduser}@{ad_realm}')
        # Run ssh command
        ssh_result = client.auth_from_client(
            f'{aduser}@{ad_realm}', 'Secret123') == 3
        client.restore_sssd_conf()
        client.clear_sssd_cache()

        # Evaluate test results
        assert su_result, "The su command failed!"
        assert ssh_result, "The ssh login failed!"

    @staticmethod
    @pytest.mark.usefixtures("sssd_domain")
    def test_0003_login_sssd_domain(multihost, create_aduser_group):
        """test_0003_login_sssd_domain

        :title: IDM-SSSD-TC: ad_provider: ad_login_attr: Verify user login
         using sssd domain name
        :id: 6b40310f-c1a8-4179-b41a-69b3eecde4ff
        :setup:
          1. Configure sssd with sssd_domain
        :steps:
          1. Run su for the user with sssd_domain.
          2. Run ssh for the user with sssd_domain.
        :expectedresults:
          1. Su command succeeds.
          2. Ssh command succeeds.
        :customerscenario: False
        """
        # Configure sssd
        multihost.client[0].service_sssd('stop')
        client = sssdTools(multihost.client[0], multihost.ad[0])
        dom_section = 'domain/sssd_domain'
        sssd_params = {
            'ldap_id_mapping': 'True',
            'debug_level': '9',
            'id_provider': 'ad',
            'use_fully_qualified_names': 'False',
            'ad_domain': multihost.ad[0].domainname.lower(),
            'ad_server': multihost.ad[0].hostname,
            'ldap_schema': 'ad',
            'ldap_user_principal': 'userPrincipalName',
        }
        client.sssd_conf(dom_section, sssd_params)
        client.sssd_conf('sssd', {'services': 'nss, pam, ssh'})
        client.clear_sssd_cache()

        # Create AD user
        (aduser, _) = create_aduser_group

        # Run su command
        su_result = client.su_success(f'{aduser}@sssd_domain')
        # Run ssh command
        ssh_result = client.auth_from_client(
            f'{aduser}@sssd_domain', 'Secret123') == 3
        # Evaluate test results
        assert su_result, "The su command failed!"
        assert ssh_result, "The ssh login failed!"

    @staticmethod
    @pytest.mark.usefixtures("sssd_domain")
    def test_0004_login_sssd_domain_fqn(multihost, create_aduser_group):
        """test_0004_login_sssd_domain_fqn

        :title: IDM-SSSD-TC: ad_provider: ad_login_attr: Verify login
         attributes with fully qualified name
        :id: 536ff920-e1c1-4948-b0ce-4fc024e1c5ba
        :setup:
          1. Configure sssd with sssd_domain and userPrincipalName
          2. Configure full_name_format to %1$s@%2$s
        :steps:
          1. Run su for the user with sssd domain.
          2. Run su for the user with fully qualified name.
          3. Run ssh for the user with sssd domain.
          4. Run ssh for the user with fully qualified name.
        :expectedresults:
          1. Su command succeeds
          2. Su command succeeds
          3. Ssh command succeeds
          4. Ssh command succeeds
        :customerscenario: False
        """
        # Configure sssd
        multihost.client[0].service_sssd('stop')
        ad_realm = multihost.ad[0].domainname.upper()
        client = sssdTools(multihost.client[0], multihost.ad[0])
        dom_section = 'domain/sssd_domain'
        sssd_params = {
            'ldap_id_mapping': 'True',
            'debug_level': '9',
            'id_provider': 'ad',
            'use_fully_qualified_names': 'True',
            'ad_domain': multihost.ad[0].domainname.lower(),
            'ad_server': multihost.ad[0].hostname,
            'ldap_schema': 'ad',
            'full_name_format': '%1$s@%2$s',
            'ldap_user_principal': 'userPrincipalName',
        }
        client.sssd_conf(dom_section, sssd_params)
        client.sssd_conf('sssd', {'services': 'nss, pam, ssh'})
        client.clear_sssd_cache()

        # Create AD user
        (aduser, _) = create_aduser_group

        # Run su command
        su_result_plain = client.su_success(f'{aduser}')
        su_result_sssd = client.su_success(f'{aduser}@sssd_domain')
        su_result_fqn = client.su_success(
            f'{aduser}@{ad_realm}')
        # Run ssh command
        ssh_result = client.auth_from_client(
            f'{aduser}@sssd_domain', 'Secret123') == 3
        ssh_result_fqn = client.auth_from_client(
            f'{aduser}@{ad_realm}', 'Secret123') == 3
        # Evaluate test results
        assert not su_result_plain, "The su command did not fail!"
        assert su_result_sssd, "The su command failed for sssd_domain!"
        assert su_result_fqn, "The su command failed for fqn!"
        assert ssh_result, "The ssh login with sssd_domain failed!"
        assert ssh_result_fqn, "The ssh login with fqn failed!"

    @staticmethod
    def test_0005_login_sssd_domain_uppercase(
            multihost, create_aduser_group):
        """test_0005_login_sssd_domain_uppercase

        :title: IDM-SSSD-TC: ad_provider: ad_login_attr: Verify login
         attributes with upper case names
        :id: 03d198fd-96bc-4ea4-9ba3-dd37bae3def5
        :setup:
          1. Configure sssd with ldap_user_principal = userPrincipalName
        :steps:
          1. Run su for the user in uppercase without domain.
          2. Run su for the user in uppercase with domain.
          3. Run ssh for the user in uppercase with domain.
        :expectedresults:
          1. Su command fails.
          2. Su command succeeds.
          2. Ssh command succeeds.
        :customerscenario: False
        """
        # Configure sssd
        multihost.client[0].service_sssd('stop')
        client = sssdTools(multihost.client[0], multihost.ad[0])
        client.backup_sssd_conf()
        dom_section = f'domain/{client.get_domain_section_name()}'
        sssd_params = {
            'ldap_id_mapping': 'True',
            'debug_level': '9',
            'id_provider': 'ad',
            'use_fully_qualified_names': 'True',
            'ad_domain': multihost.ad[0].domainname.lower(),
            'ad_server': multihost.ad[0].hostname,
            'ldap_schema': 'ad',
            'ldap_user_principal': 'userPrincipalName',
        }
        client.sssd_conf(dom_section, sssd_params)
        client.sssd_conf('sssd', {'services': 'nss, pam, ssh'})
        client.clear_sssd_cache()

        # Create AD user
        (aduser, _) = create_aduser_group

        # Run su command
        su_result_upper = client.su_success(f'{aduser.upper()}')
        su_result_fqn = client.su_success(
            f'{aduser.upper()}@{multihost.ad[0].domainname}')

        # Run ssh command
        ssh = pxssh.pxssh(options={"StrictHostKeyChecking": "no",
                          "UserKnownHostsFile": "/dev/null"})
        ssh.force_password = True
        ssh_result = True
        try:
            ssh.login(
                multihost.client[0].sys_hostname,
                f'{aduser.upper()}@{multihost.ad[0].domainname}', 'Secret123')
            ssh.prompt(timeout=5)
            ssh.logout()
        except pxssh.ExceptionPxssh:
            ssh_result = False

        client.restore_sssd_conf()
        client.clear_sssd_cache()

        # Evaluate test results
        assert not su_result_upper, "The su command did not fail as expected!"
        assert su_result_fqn, "The su command failed for fqn!"
        assert ssh_result, "The ssh login failed!"



""" Automation of proxy provider suite

:requirement: IDM-SSSD-REQ : Proxy Provider
:casecomponent: sssd
:subsystemteam: sst_idm_sssd
:upstream: yes
:status: approved
"""
from __future__ import print_function
import pytest
from sssd.testlib.common.utils import sssdTools
from sssd.testlib.common.ssh2_python import check_login_client_bool, check_login_client


def execute_cmd(multihost, command):
    """ Execute command on client """
    cmd = multihost.client[0].run_command(command)
    return cmd


@pytest.mark.usefixtures('setup_sssd_krb',
                         'create_posix_usersgroups',
                         'install_nslcd',
                         'sssdproxyldap',
                         'sssdproxyldap_test')
@pytest.mark.tier1_3
class TestProxy(object):
    """
    This is test case class for ldap proxy suite
    """
    @staticmethod
    @pytest.mark.usefixtures('backupsssdconf')
    def test_proxy_lookup(multihost):
        """
        :title: Proxy lookup and kerberos auth
        :id: 5c4c55b8-0cac-47d0-aa23-d057b790e18e
        :steps:
          1. Check ldap user has access to client machine
          2. Proxy lookup and kerberos auth
        :expectedresults:
          1. Should succeed
          2. Should succeed
        """
        # proxy lookup and kerberos auth
        tools = sssdTools(multihost.client[0])
        tools.clear_sssd_cache()
        ssh = check_login_client_bool(multihost, "foo2@example1", 'Secret123')
        assert ssh, 'foo2@example1 is not able to login.'
        assert "home/foo2:/bin/bash" in execute_cmd(multihost,
                                                    "getent -s "
                                                    "ldap passwd "
                                                    "foo2").stdout_text

    @staticmethod
    def test_expired_password(multihost):
        """
        :title: Change expired password
        :id: 60de31d3-82e8-4f35-999d-5f9f15e0caed
        :steps:
          1. Change expired password
        :expectedresults:
          1. Should succeed
        """
        tools = sssdTools(multihost.client[0])
        # Change expired password
        multihost.master[0].run_command("kadmin.local "
                                        "-q 'modprinc -pwexpire "
                                        "now foo2@example1'")
        tools.clear_sssd_cache()
        multihost.client[0].run_command("yum install -y expect",
                                        raiseonerr=False)
        execute_cmd(multihost, "sh /tmp/sssdproxyldap.sh")
        multihost.master[0].run_command("kadmin.local -q "
                                        "'addprinc -pw Secret123 "
                                        "foo2@example1'")

    @staticmethod
    def test_server_access(multihost):
        """
        :title: Proxy server access
        :id: 5fe5839b-fbfb-48be-87de-18c1b0de209c
        :steps:
          1. Block server access
          2. User should not have access
          3. Unblock server access
          4. User should have access
        :expectedresults:
          1. Should succeed
          2. Should succeed
          3. Should succeed
          4. Should succeed
        """
        tools = sssdTools(multihost.client[0])
        tools.clear_sssd_cache()
        ssh = check_login_client_bool(multihost, "foo2@example1", 'Secret123')
        # block_server_access
        execute_cmd(multihost, "systemctl start firewalld")
        execute_cmd(multihost, f"firewall-cmd --direct "
                               f"--add-rule ipv4 filter "
                               f"OUTPUT 0 -d {multihost.master[0].ip} "
                               f"-j DROP")
        try:
            with pytest.raises(Exception):
                check_login_client(multihost, "foo2@example1", 'Secret123')
            # unblock_server_access
            execute_cmd(multihost, "firewall-cmd  --reload")
            execute_cmd(multihost, "systemctl stop firewalld")
            tools.clear_sssd_cache()
            ssh2 = check_login_client_bool(multihost, "foo2@example1", 'Secret123')
            assert ssh, 'foo2@example1 is not able to login.'
            assert ssh2, 'foo2@example1 is not able to login.'
        except Exception:
            # unblock_server_access
            execute_cmd(multihost, "firewall-cmd  --reload")
            execute_cmd(multihost, "systemctl stop firewalld")
            tools.clear_sssd_cache()
            pytest.fail("foo2@example1 was able to login")



#
# Module for simulation of utility "id" from coreutils
#
# Copyright (c) 2015 Red Hat, Inc.
# Author: Lukas Slebodnik <lslebodn@redhat.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
import pwd
import grp
from ctypes import (c_int, c_char, c_uint32, c_long, c_char_p,
                    POINTER, pointer)
from sssd_nss import NssReturnCode, nss_sss_ctypes_loader


def call_sssd_initgroups(user, gid):
    """
    Function will initialize the supplementary group access list
    for given user. It will gather groups only provided by sssd.

    Arguments are the same as for C function initgroups
    @param string user name of user
    @param int gid the additional gid will be also added to the list.

    @return (int, int, List[int]) (err, errno, gids)
        gids should contain user group IDs if err is NssReturnCode.SUCCESS
        otherwise errno will contain non-zero value.
    """
    func = nss_sss_ctypes_loader('_nss_sss_initgroups_dyn')

    func.restype = c_int
    func.argtypes = [POINTER(c_char), c_uint32, POINTER(c_long),
                     POINTER(c_long), POINTER(POINTER(c_uint32)), c_long,
                     POINTER(c_int)]

    start = POINTER(c_long)(c_long(0))
    size = POINTER(c_long)(c_long(0))
    groups = POINTER(c_uint32)()
    p_groups = pointer(groups)
    limit = c_long(-1)
    errno = POINTER(c_int)(c_int(0))

    res = func(c_char_p(user.encode('utf-8)')), c_uint32(gid), start, size,
               p_groups, limit, errno)

    gids = []
    if res == NssReturnCode.SUCCESS:
        gids_count = size[0]
        assert gids_count > 0, "_nss_sss_initgroups_dyn should return " \
                               "one gid"

        for i in range(0, gids_count):
            gids.append(int(p_groups.contents[i]))

        # add primary group if missing
        if gid not in gids:
            gids.append(gid)

    return (int(res), errno[0], gids)


def get_user_gids(user):
    """
    Function will initialize the supplementary group access list
    for given user. It will gather groups only provided by sssd.

    Arguments are the same as for C function initgroups
    @param string user name of user

    @return (int, int, List[int]) (err, errno, gids)
        gids should contain user group IDs if err is NssReturnCode.SUCCESS
        otherwise errno will contain non-zero value.
    """
    pwd_user = pwd.getpwnam(user)
    uid = pwd_user.pw_uid
    gid = pwd_user.pw_gid

    user = pwd.getpwuid(uid).pw_name

    return call_sssd_initgroups(user, gid)


def gid_to_str(gid):
    """
    Function will map numeric GID into names.
    If there isn't a group for GID (getgrgid failed)
    then the function will return decimal representation of ID.

    @param int gid ID of groups which should be converted to string.
    @return string name of group with requested ID or decimal
                   representation of ID
    """
    try:
        return grp.getgrgid(gid).gr_name
    except KeyError:
        return str(gid)


def get_user_groups(user):
    """
    Function will initialize the supplementary group access list
    for given user. It will gather groups only provided by sssd.

    Arguments are the same as for C function initgroups
    @param string user name of user

    @return (int, int, List[string]) (err, errno, groups)
        groups should contain names of user groups
        if err is NssReturnCode.SUCCESS
        otherwise errno will contain non-zero value.
    """
    (res, errno, gids) = get_user_gids(user)
    groups = []

    if res == NssReturnCode.SUCCESS:
        groups = [gid_to_str(gid) for gid in gids]

    return (res, errno, groups)



"""
Local Override "sss_override" Tests.

Note: IPA does not support this feature because, it manages user and group overrides using ID views on the IPA master.

:requirement:  IDM-SSSD-TC: ldap_provider: local_overrides
"""

from __future__ import annotations

import pytest
from sssd_test_framework.roles.client import Client
from sssd_test_framework.roles.generic import GenericProvider
from sssd_test_framework.topology import KnownTopology


@pytest.mark.importance("medium")
@pytest.mark.topology(KnownTopology.LDAP)
@pytest.mark.topology(KnownTopology.AD)
@pytest.mark.topology(KnownTopology.Samba)
def test_sss_overrides__overriding_username_and_posix_attributes(client: Client, provider: GenericProvider):
    """
    :title: Locally overriding the name and POSIX attributes of a user
    :setup:
        1. Create POSIX user "user1"
        2. Configure SSSD with "ldap_id_mapping = false" and start SSSD
        3. Create local override for "user1"
        4. Restart SSSD, this is necessary to enable local overrides
    :steps:
        1. Authenticate as "user1", then as the override name, use the short and fully qualified name
        2. Lookup user by the overridden name, check the uid and gid
        3. Override user's uid, gid and homedir and lookup user by both names, check the uid and gid
    :expectedresults:
        1. Users logins are successful
        2. User is found and uid and gid match original values
        3. User is found using both names and uid, gid and homedir match the new values
    :customerscenario: False
    """
    provider.user("user1").add(
        uid=999011, gid=999011, home="/home/user1", gecos="user", shell="/bin/bash", password="Secret123"
    )

    client.sssd.domain["ldap_id_mapping"] = "False"
    client.sssd.start()

    client.sss_override.user("user1").add(name="o-user1")

    client.sssd.restart()

    assert client.auth.ssh.password("user1", "Secret123"), "Failed login!"
    assert client.auth.ssh.password(
        f"user1@{client.sssd.default_domain}", "Secret123"
    ), "Fully qualified name failed login!"

    assert client.auth.ssh.password("o-user1", "Secret123"), "Override name failed login!"
    assert client.auth.ssh.password(
        f"o-user1@{client.sssd.default_domain}", "Secret123"
    ), "Override fully qualified name failed login!"

    result = client.tools.getent.passwd("o-user1")
    assert result is not None, "User not found by override name!"
    assert result.uid == 999011, "User's uid does not match original value!"
    assert result.gid == 999011, "User's gid does not match original value!"

    client.sss_override.user("user1").add(name="o-user1", uid=999999, gid=888888, home="/home/o-user1")

    result = client.tools.getent.passwd("user1")
    assert result is not None, "User not found!"
    assert result.uid == 999999, "User's uid does not match override value!"
    assert result.gid == 888888, "User's gid does not match override value!"
    assert result.home == "/home/o-user1", "User's homedir does not match override value!"

    result = client.tools.getent.passwd("o-user1")
    assert result is not None, "User not found by override name!"
    assert result.uid == 999999, "Local override uid does not match override value!"
    assert result.gid == 888888, "Local override gid does not match override value!"
    assert result.home == "/home/o-user1", "User's override name homedir does not match override value!"


@pytest.mark.importance("medium")
@pytest.mark.topology(KnownTopology.LDAP)
@pytest.mark.topology(KnownTopology.AD)
@pytest.mark.topology(KnownTopology.Samba)
def test_sss_overrides__overriding_group_name_and_gid(client: Client, provider: GenericProvider):
    """
    :title: Locally overriding the name and GID of a group
    :setup:
        1. Create POSIX group "group1"
        2. Configure SSSD with "ldap_id_mapping = false" and start SSSD
        3. Create local override for "group1"
        4. Restart SSSD, this is necessary to enable local overrides
    :steps:
        1. Lookup group name and overridden name and check gid
        2. Override group gid to a new value, lookup group name by both names and check gid
    :expectedresults:
        1. Groups are found and gid match original values
        2. Groups are found and gid matches new overridden value
    :customerscenario: False
    """
    provider.group("group1").add(gid=999999)
    client.sssd.domain["ldap_id_mapping"] = "False"
    client.sssd.start()

    group = client.sss_override.group("group1")

    group.add(name="o-group1")

    client.sssd.restart()

    result = client.tools.getent.group("group1")
    assert result is not None, "Group not found!"
    assert result.gid == 999999, "Group gid does not match original value!"
    result = client.tools.getent.group("o-group1")
    assert result is not None, "Group not found by override name!"
    assert result.gid == 999999, "Local override gid does match original value! "

    group.add(name="o-group1", gid=888888)

    result = client.tools.getent.group("group1")
    assert result is not None, "Group not found!"
    assert result.gid == 888888, "Group gid does not match override value!"

    result = client.tools.getent.group("o-group1")
    assert result is not None, "Group not found by override name!"
    assert result.gid == 888888, "Local override gid does not match override value!"


@pytest.mark.importance("medium")
@pytest.mark.topology(KnownTopology.LDAP)
@pytest.mark.topology(KnownTopology.AD)
@pytest.mark.topology(KnownTopology.Samba)
def test_sss_overrides__root_uid_gid_cannot_be_used(client: Client, provider: GenericProvider):
    """
    :title: Root user UID/GID cannot be overridden
    :setup:
        1. Create POSIX user "user1" and "root"
        2. Configure SSSD with "use_fully_qualified_names = false" and start SSSD
        3. Create local override for "user1" and name it "root" root, set the uid/gid to '0'
        4. Restart SSSD, this is necessary to enable local overrides
    :steps:
        1. Lookup the root user and check his uid and gid
        2. Lookup the root user and use the sss service
        3. Lookup user and check his uid and gid
    :expectedresults:
        1. The root user uid and gid has not been modified
        2. root user is not found
        3. User found and uid and gid is not roots
    :customerscenario: False
    """
    provider.user("user1").add(uid=999011, gid=999011)
    provider.user("root").add(uid=999012, gid=999012)
    client.sssd.domain["ldap_id_mapping"] = "False"
    client.sssd.domain["use_fully_qualified_names"] = "False"
    client.sssd.start()

    client.sss_override.user("user1").add(name="root", uid=0, gid=0)

    client.sssd.restart()

    result = client.tools.getent.passwd("root")
    assert result is not None, "root user not found!"
    assert result.uid == 0, "root uid is not 0"
    assert result.gid == 0, "root gid is not 0"

    # Root should be filtered out from any service other than files
    assert client.tools.getent.passwd("root", service="sss") is None, "root user is found!"

    result = client.tools.getent.passwd("user1")
    assert result is not None, "user1 not found!"
    assert result.uid != 0, "User uid is 0!"
    assert result.gid != 0, "User gid is 0!"


@pytest.mark.importance("medium")
@pytest.mark.topology(KnownTopology.LDAP)
@pytest.mark.topology(KnownTopology.AD)
@pytest.mark.topology(KnownTopology.Samba)
def test_sss_overrides__export_then_import_user_and_group_override_data(client: Client, provider: GenericProvider):
    """
        :title: Export and import the user and group local override data
    :setup:
        1. Create posix user "user1" with posix attributes defined
        2. Start SSSD
    :steps:
        1. Override user "user1" to "o-user1"
        2. Restart SSSD, this is necessary to enable local overrides
        3. Override group "group1" to "o-group1"
        4. Export user and group local overrides data to a file
        5. Delete overrides and restart SSSD
        6. Import user and group local overrides data and restart SSSD
        7. Search for user and group local overrides
    :expectedresults:
        1. User local override has been created
        2. SSSD has been restarted successfully
        3. Group local override has been created
        4. Local overrides user and group data is exported to a file
        5. SSSD is restarted and overrides data is gone
        6. User and group local override data has been imported from the export
        7. User and group local overrides is found
    :customerscenario: False
    :requirement: IDM-SSSD-TC: ldap_provider: local_overrides: import export user override
    """
    provider.user("user1").add(
        uid=999011, gid=999011, home="/home/user1", gecos="user", shell="/bin/bash", password="Secret123"
    )
    provider.group("group1").add(gid=999999)
    client.sssd.start()

    user = client.sss_override.user("user1")
    group = client.sss_override.group("group1")
    override = client.sss_override

    user.add(name="o-user1")
    client.sssd.restart()
    group.add(name="o-group1")

    # local_override.export_data(users=None) exports all user data
    override.export_data()
    user.delete()
    group.delete()

    client.sssd.restart()
    assert not client.sss_override.user("user1").get()
    assert not client.sss_override.group("group1").get()

    assert (
        len(
            client.ldb.search(
                f"/var/lib/sss/db/cache_{client.sssd.default_domain}.ldb",
                f"cn={client.sssd.default_domain},cn=sysdb",
                filter="objectClass=userOverride",
            ).items()
        )
        < 1
    ), "Override is not empty!"

    client.sss_override.import_data()
    client.sssd.restart()

    assert client.sss_override.user("user1").get(["name"]) == {"name": ["o-user1"]}
    assert client.sss_override.group("group1").get(["name"]) == {"name": ["o-group1"]}


@pytest.mark.importance("medium")
@pytest.mark.ticket(bz=1254184)
@pytest.mark.topology(KnownTopology.LDAP)
@pytest.mark.topology(KnownTopology.AD)
@pytest.mark.topology(KnownTopology.Samba)
def test_sss_overrides__use_fully_qualified_names_is_true(client: Client, provider: GenericProvider):
    """
    :title: Overriding user when use_fully_qualified_names is true
    :setup:
        1. Create user "user1"
        2. Edit SSSD configuration and set "use_fully_qualified_names" = True
        3. Start SSSD
        4. Create override for "user1"
        5. Restart SSSD, this is necessary to enable local overrides
    :steps:
        1. Login with the username and overridden name
        2. Login with the fully qualified username and overridden name
    :expectedresults:
        1. Logins fail
        2. Login succeed
    :customerscenario: False
    """
    provider.user("user1").add()
    client.sssd.domain["use_fully_qualified_names"] = "True"
    client.sssd.start()

    client.sss_override.user(f"user1@{client.sssd.default_domain}").add(name="o-user1")

    client.sssd.restart()

    assert not client.auth.ssh.password("user1", "Secret123"), "User logged in!"
    assert not client.auth.ssh.password("o-user1", "Secret123"), "User logged in!"
    assert client.auth.ssh.password(f"user1@{client.sssd.default_domain}", "Secret123"), "Login failed!"
    assert client.auth.ssh.password(f"o-user1@{client.sssd.default_domain}", "Secret123"), "Login failed!"
